"""Agent Function manager for handling agent functions operations."""

from typing import TYPE_CHECKING, Any, Dict, List, Optional

from ..types import (
    AgentFunction,
    AgentFunctionChatAction,
    AuthorisationType,
    FunctionRequestType,
    FunctionScope,
    ListResponse,
)
from ..utils import convert_kwargs_to_api
from .field_mappings import AGENT_FUNCTION_FIELD_MAPPINGS, AGENT_FUNCTION_EXPLICIT_PARAMS

if TYPE_CHECKING:
    from ..client import ToothFairyClient


class AgentFunctionManager:
    """Manager for agent functions operations.

    This manager provides methods to create, update, and manage agent functions.

    Example:
        >>> client = ToothFairyClient(api_key="...", workspace_id="...")
        >>> agent_function = client.agent_functions.create(...)
    """

    def __init__(self, client: "ToothFairyClient"):
        """Initialize the AgentFunctionManager.

        Args:
            client: The ToothFairyClient instance.
        """
        self._client = client

    def create(
        self,
        name: str,
        description: str,
        url: Optional[str] = None,
        request_type: Optional[FunctionRequestType] = None,
        authorisation_type: AuthorisationType = "none",
        authorisation_key: Optional[str] = None,
        authorisation_id: Optional[str] = None,
        parameters: Optional[Dict[str, Any]] = None,
        model: Optional[str] = None,
        depends_on_before_execution: Optional[List[str]] = None,
        depends_on_after_execution: Optional[List[str]] = None,
        params_as_path: Optional[bool] = None,
        scope: Optional[FunctionScope] = None,
        json_path_extractor: Optional[str] = None,
        headers: Optional[Dict[str, Any]] = None,
        static_args: Optional[Dict[str, Any]] = None,
        custom_execution_code: Optional[str] = None,
        is_mcp_server: Optional[bool] = None,
        is_callback_fn: Optional[bool] = None,
        is_database_script: Optional[bool] = None,
        is_html_fn: Optional[bool] = None,
        is_chat_fn: Optional[bool] = None,
        is_hand_off_fn: Optional[bool] = None,
        is_code_template: Optional[bool] = None,
        is_agent_skill: Optional[bool] = None,
        code_execution_environment_id: Optional[str] = None,
        on_fn_call: Optional[Dict[str, Any]] = None,
        hand_off_agent: Optional[str] = None,
        hand_off_to_original_agent_on_completion: Optional[bool] = None,
        hand_off_to_original_agent_on_failure: Optional[bool] = None,
        hand_off_agent_on_completion: Optional[str] = None,
        hand_off_agent_on_failure: Optional[str] = None,
        agent_skill_script: Optional[str] = None,
        chat_script: Optional[str] = None,
        chat_action: Optional[AgentFunctionChatAction] = None,
        html_url: Optional[str] = None,
        html_script: Optional[str] = None,
        is_html_redirect_fn: Optional[bool] = None,
        html_form_on_successful_submit_message: Optional[str] = None,
        html_form_on_failed_submit_message: Optional[str] = None,
        db_schema_metadata: Optional[Dict[str, Any]] = None,
        pre_execution_script: Optional[str] = None,
        post_execution_script: Optional[str] = None,
        db_script: Optional[str] = None,
        db_connection_id: Optional[str] = None,
        pre_execution_fn_id: Optional[str] = None,
        pre_execution_mapping: Optional[Dict[str, Any]] = None,
        oauth_connection_id: Optional[str] = None,
        handed_off_to_human_on_call: Optional[bool] = None,
        graphql_query: Optional[str] = None,
        graphql_mutation: Optional[str] = None,
        graphql_operation_name: Optional[str] = None,
        users_to_hand_off_to: Optional[List[str]] = None,
        is_global: Optional[bool] = None,
    ) -> AgentFunction:
        """Create a new agent function.

        Args:
            name: Human-readable name of the function.
            description: Detailed description of function purpose (required).
            url: Endpoint URL for the function.
            request_type: HTTP method (GET, POST, PUT, PATCH, DELETE).
            authorisation_type: Authentication method (none, bearer, apikey, basic, oauth2).
            authorisation_key: Authentication key or token.
            authorisation_id: ID of authorisation configuration.
            parameters: Function parameters schema (AWSJSON).
            model: Model to use for function execution.
            depends_on_before_execution: List of function IDs to execute before this function.
            depends_on_after_execution: List of function IDs to execute after this function.
            params_as_path: Whether to pass parameters as path parameters.
            scope: Function scope (workspace, global).
            json_path_extractor: JSONPath expression to extract data from response.
            headers: HTTP headers for the request (AWSJSON).
            static_args: Static arguments passed to function (AWSJSON).
            custom_execution_code: Custom code for function execution.
            is_mcp_server: Whether this is an MCP server.
            is_callback_fn: Whether this is a callback function.
            is_database_script: Whether this is a database script.
            is_html_fn: Whether this is an HTML function.
            is_chat_fn: Whether this is a chat function.
            is_hand_off_fn: Whether this is a hand-off function.
            is_code_template: Whether this is a code template.
            is_agent_skill: Whether this is an agent skill.
            code_execution_environment_id: ID of code execution environment.
            on_fn_call: Configuration for on function call (AWSJSON).
            hand_off_agent: ID of agent to hand off to.
            hand_off_to_original_agent_on_completion: Return to original agent on completion.
            hand_off_to_original_agent_on_failure: Return to original agent on failure.
            hand_off_agent_on_completion: ID of agent to hand off to on completion.
            hand_off_agent_on_failure: ID of agent to hand off to on failure.
            agent_skill_script: Script for agent skill execution.
            chat_script: Script for chat function.
            chat_action: Action for chat function (reply, silent, redirect).
            html_url: URL for HTML function.
            html_script: HTML script content.
            is_html_redirect_fn: Whether this is an HTML redirect function.
            html_form_on_successful_submit_message: Message on successful form submit.
            html_form_on_failed_submit_message: Message on failed form submit.
            db_schema_metadata: Database schema metadata (AWSJSON).
            pre_execution_script: Script to run before execution.
            post_execution_script: Script to run after execution.
            db_script: Database script content.
            db_connection_id: ID of database connection.
            pre_execution_fn_id: ID of pre-execution function.
            pre_execution_mapping: Mapping for pre-execution (AWSJSON).
            oauth_connection_id: ID of OAuth connection.
            handed_off_to_human_on_call: Whether to hand off to human on call.
            graphql_query: GraphQL query string.
            graphql_mutation: GraphQL mutation string.
            graphql_operation_name: GraphQL operation name.
            users_to_hand_off_to: List of user IDs to hand off to.
            is_global: Whether this function is global.

        Returns:
            The created AgentFunction object.
        """
        data: Dict[str, Any] = {
            "name": name,
            "description": description,
            "authorisationType": authorisation_type,
        }

        if url is not None:
            data["url"] = url
        if request_type is not None:
            data["requestType"] = request_type
        if authorisation_key is not None:
            data["authorisationKey"] = authorisation_key
        if authorisation_id is not None:
            data["authorisationID"] = authorisation_id
        if parameters is not None:
            data["parameters"] = parameters
        if model is not None:
            data["model"] = model
        if depends_on_before_execution is not None:
            data["dependsOnBeforeExecution"] = depends_on_before_execution
        if depends_on_after_execution is not None:
            data["dependsOnAfterExecution"] = depends_on_after_execution
        if params_as_path is not None:
            data["paramsAsPath"] = params_as_path
        if scope is not None:
            data["scope"] = scope
        if json_path_extractor is not None:
            data["jsonPathExtractor"] = json_path_extractor
        if headers is not None:
            data["headers"] = headers
        if static_args is not None:
            data["staticArgs"] = static_args
        if custom_execution_code is not None:
            data["customExecutionCode"] = custom_execution_code
        if is_mcp_server is not None:
            data["isMCPServer"] = is_mcp_server
        if is_callback_fn is not None:
            data["isCallbackFn"] = is_callback_fn
        if is_database_script is not None:
            data["isDatabaseScript"] = is_database_script
        if is_html_fn is not None:
            data["isHtmlFn"] = is_html_fn
        if is_chat_fn is not None:
            data["isChatFn"] = is_chat_fn
        if is_hand_off_fn is not None:
            data["isHandOffFn"] = is_hand_off_fn
        if is_code_template is not None:
            data["isCodeTemplate"] = is_code_template
        if is_agent_skill is not None:
            data["isAgentSkill"] = is_agent_skill
        if code_execution_environment_id is not None:
            data["codeExecutionEnvironmentID"] = code_execution_environment_id
        if on_fn_call is not None:
            data["onFnCall"] = on_fn_call
        if hand_off_agent is not None:
            data["handOffAgent"] = hand_off_agent
        if hand_off_to_original_agent_on_completion is not None:
            data["handOffToOriginalAgentOnCompletion"] = hand_off_to_original_agent_on_completion
        if hand_off_to_original_agent_on_failure is not None:
            data["handOffToOriginalAgentOnFailure"] = hand_off_to_original_agent_on_failure
        if hand_off_agent_on_completion is not None:
            data["handOffAgentOnCompletion"] = hand_off_agent_on_completion
        if hand_off_agent_on_failure is not None:
            data["handOffAgentOnFailure"] = hand_off_agent_on_failure
        if agent_skill_script is not None:
            data["agentSkillScript"] = agent_skill_script
        if chat_script is not None:
            data["chatScript"] = chat_script
        if chat_action is not None:
            data["chatAction"] = chat_action
        if html_url is not None:
            data["htmlUrl"] = html_url
        if html_script is not None:
            data["htmlScript"] = html_script
        if is_html_redirect_fn is not None:
            data["isHtmlRedirectFn"] = is_html_redirect_fn
        if html_form_on_successful_submit_message is not None:
            data["htmlFormOnSuccessfulSubmitMessage"] = html_form_on_successful_submit_message
        if html_form_on_failed_submit_message is not None:
            data["htmlFormOnFailedSubmitMessage"] = html_form_on_failed_submit_message
        if db_schema_metadata is not None:
            data["dbSchemaMetadata"] = db_schema_metadata
        if pre_execution_script is not None:
            data["preExecutionScript"] = pre_execution_script
        if post_execution_script is not None:
            data["postExecutionScript"] = post_execution_script
        if db_script is not None:
            data["dbScript"] = db_script
        if db_connection_id is not None:
            data["dbConnectionID"] = db_connection_id
        if pre_execution_fn_id is not None:
            data["preExecutionFnID"] = pre_execution_fn_id
        if pre_execution_mapping is not None:
            data["preExecutionMapping"] = pre_execution_mapping
        if oauth_connection_id is not None:
            data["OAuthConnectionID"] = oauth_connection_id
        if handed_off_to_human_on_call is not None:
            data["handedOffToHumanOnCall"] = handed_off_to_human_on_call
        if graphql_query is not None:
            data["graphqlQuery"] = graphql_query
        if graphql_mutation is not None:
            data["graphqlMutation"] = graphql_mutation
        if graphql_operation_name is not None:
            data["graphqlOperationName"] = graphql_operation_name
        if users_to_hand_off_to is not None:
            data["usersToHandOffTo"] = users_to_hand_off_to
        if is_global is not None:
            data["isGlobal"] = is_global

        response = self._client.request("POST", "/function/create", data=data)
        return AgentFunction.from_dict(response)

    def update(
        self,
        agent_function_id: str,
        **kwargs: Any,
    ) -> AgentFunction:
        """Update a agent_function.
        
        Fields can be passed using snake_case and will be converted to API format.
        You can also pass camelCase field names directly to bypass conversion.

        Args:
            agent_function_id: ID of the agent_function to update.
            **kwargs: Fields to update (snake_case or camelCase).

        Returns:
            The updated AgentFunction object.
        """
        data: Dict[str, Any] = {"id": agent_function_id}
        data.update(convert_kwargs_to_api(
            kwargs,
            AGENT_FUNCTION_FIELD_MAPPINGS,
            AGENT_FUNCTION_EXPLICIT_PARAMS
        ))
        response = self._client.request("POST", "/function/update", data=data)
        return AgentFunction.from_dict(response)

    def delete(self, agent_function_id: str) -> Dict[str, bool]:
        """Delete a agent_function.

        Args:
            agent_function_id: ID of the agent_function to delete.

        Returns:
            A dictionary with success status.
        """
        self._client.request("DELETE", f"/function/delete/{agent_function_id}")
        return {"success": True}

    def get(self, agent_function_id: str) -> AgentFunction:
        """Get a agent_function by ID.

        Args:
            agent_function_id: ID of the agent_function to retrieve.

        Returns:
            The AgentFunction object.
        """
        response = self._client.request("GET", f"/function/get/{agent_function_id}")
        return AgentFunction.from_dict(response)

    def list(
        self,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
    ) -> ListResponse:
        """List all agent functions.

        Args:
            limit: Maximum number of agent functions to return.
            offset: Number of agent functions to skip.

        Returns:
            A ListResponse containing the agent functions.
        """
        params: Dict[str, Any] = {}
        if limit is not None:
            params["limit"] = limit
        if offset is not None:
            params["offset"] = offset

        response = self._client.request("GET", "/function/list", params=params)

        items = []
        if isinstance(response, list):
            items = [AgentFunction.from_dict(item) for item in response]
        elif isinstance(response, dict):
            items = [AgentFunction.from_dict(item) for item in response.get("items", [])]

        return ListResponse(items=items)

    def search(self, search_term: str) -> List[AgentFunction]:
        """Search agent functions by name.

        Args:
            search_term: Term to search for in function names.

        Returns:
            A list of matching AgentFunction objects.
        """
        all_functions = self.list()
        search_lower = search_term.lower()
        return [func for func in all_functions.items if search_lower in func.name.lower()]



"""Tests for field mapping utilities."""

import pytest
from toothfairyai.utils import to_camel_case, convert_kwargs_to_api
from toothfairyai.agents.field_mappings import AGENT_FIELD_MAPPINGS, AGENT_EXPLICIT_PARAMS
from toothfairyai.agent_functions.field_mappings import (
    AGENT_FUNCTION_FIELD_MAPPINGS,
    AGENT_FUNCTION_EXPLICIT_PARAMS,
)


class TestToCamelCase:
    """Tests for to_camel_case function."""

    def test_simple_snake_case(self):
        """Test simple snake_case conversion."""
        assert to_camel_case("json_output_structure") == "jsonOutputStructure"
        assert to_camel_case("enable_json_output") == "enableJsonOutput"
        assert to_camel_case("max_tokens") == "maxTokens"

    def test_single_word(self):
        """Test single word passes through unchanged."""
        assert to_camel_case("label") == "label"
        assert to_camel_case("description") == "description"
        assert to_camel_case("temperature") == "temperature"

    def test_already_camel_case(self):
        """Test already camelCase passes through unchanged."""
        assert to_camel_case("jsonOutputStructure") == "jsonOutputStructure"
        assert to_camel_case("enableJsonOutput") == "enableJsonOutput"

    def test_multiple_underscores(self):
        """Test multiple underscores."""
        assert to_camel_case("hand_off_to_original_agent_on_completion") == "handOffToOriginalAgentOnCompletion"

    def test_empty_string(self):
        """Test empty string."""
        assert to_camel_case("") == ""

    def test_trailing_underscore(self):
        """Test trailing underscore."""
        assert to_camel_case("test_") == "test"


class TestConvertKwargsToApi:
    """Tests for convert_kwargs_to_api function."""

    def test_standard_conversion(self):
        """Test standard snake_case → camelCase conversion."""
        kwargs = {"json_output_structure": {"type": "object"}}
        result = convert_kwargs_to_api(kwargs, {})
        assert result == {"jsonOutputStructure": {"type": "object"}}

    def test_exception_mapping(self):
        """Test exception mapping overrides standard conversion."""
        kwargs = {"has_ner": True}
        result = convert_kwargs_to_api(kwargs, AGENT_FIELD_MAPPINGS)
        assert result == {"hasNER": True}

    def test_workspace_id_mapping(self):
        """Test workspace_id → workspaceid (lowercase) mapping."""
        kwargs = {"workspace_id": "test-id"}
        result = convert_kwargs_to_api(kwargs, AGENT_FIELD_MAPPINGS)
        assert result == {"workspaceid": "test-id"}

    def test_explicit_params_skipped(self):
        """Test explicit params are skipped."""
        kwargs = {"label": "test", "description": "test desc", "json_output_structure": {}}
        result = convert_kwargs_to_api(kwargs, AGENT_FIELD_MAPPINGS, {"label", "description"})
        assert result == {"jsonOutputStructure": {}}

    def test_mixed_kwargs(self):
        """Test mixed snake_case and camelCase kwargs."""
        kwargs = {
            "json_output_structure": {},
            "enableJsonOutput": True,
            "has_ner": False,
        }
        result = convert_kwargs_to_api(kwargs, AGENT_FIELD_MAPPINGS)
        assert result == {
            "jsonOutputStructure": {},
            "enableJsonOutput": True,
            "hasNER": False,
        }

    def test_unknown_field_fallback(self):
        """Test unknown field uses standard camelCase fallback."""
        kwargs = {"unknown_field": "value"}
        result = convert_kwargs_to_api(kwargs, AGENT_FIELD_MAPPINGS)
        assert result == {"unknownField": "value"}

    def test_empty_kwargs(self):
        """Test empty kwargs."""
        result = convert_kwargs_to_api({}, AGENT_FIELD_MAPPINGS)
        assert result == {}


class TestAgentFieldMappings:
    """Tests for Agent field mappings."""

    def test_id_suffix_uppercase(self):
        """Test ID suffix mappings (uppercase)."""
        assert AGENT_FIELD_MAPPINGS["parent_agent_id"] == "parentAgentID"
        assert AGENT_FIELD_MAPPINGS["parent_chat_id"] == "parentChatID"
        assert AGENT_FIELD_MAPPINGS["cloned_agent_id"] == "clonedAgentID"

    def test_id_suffix_lowercase_exception(self):
        """Test lowercase Id exception."""
        assert AGENT_FIELD_MAPPINGS["sip_dispatch_rule_id"] == "sipDispatchRuleId"

    def test_uppercase_acronyms(self):
        """Test uppercase acronym mappings."""
        assert AGENT_FIELD_MAPPINGS["has_ner"] == "hasNER"
        assert AGENT_FIELD_MAPPINGS["agentic_rag"] == "agenticRAG"
        assert AGENT_FIELD_MAPPINGS["allow_external_api"] == "allowExternalAPI"
        assert AGENT_FIELD_MAPPINGS["is_mfa_enabled"] == "isMFAEnabled"

    def test_explicit_params_set(self):
        """Test explicit params set contains expected values."""
        assert "label" in AGENT_EXPLICIT_PARAMS
        assert "description" in AGENT_EXPLICIT_PARAMS
        assert "temperature" in AGENT_EXPLICIT_PARAMS
        assert "max_tokens" in AGENT_EXPLICIT_PARAMS
        assert "max_history" in AGENT_EXPLICIT_PARAMS
        assert "top_k" in AGENT_EXPLICIT_PARAMS
        assert "doc_top_k" in AGENT_EXPLICIT_PARAMS
        assert "charting" in AGENT_EXPLICIT_PARAMS


class TestAgentFunctionFieldMappings:
    """Tests for AgentFunction field mappings."""

    def test_id_suffix_uppercase(self):
        """Test ID suffix mappings (uppercase)."""
        assert AGENT_FUNCTION_FIELD_MAPPINGS["authorisation_id"] == "authorisationID"
        assert AGENT_FUNCTION_FIELD_MAPPINGS["code_execution_environment_id"] == "codeExecutionEnvironmentID"
        assert AGENT_FUNCTION_FIELD_MAPPINGS["db_connection_id"] == "dbConnectionID"
        assert AGENT_FUNCTION_FIELD_MAPPINGS["pre_execution_fn_id"] == "preExecutionFnID"

    def test_oauth_uppercase(self):
        """Test OAuth uppercase mapping."""
        assert AGENT_FUNCTION_FIELD_MAPPINGS["oauth_connection_id"] == "OAuthConnectionID"

    def test_mcp_uppercase(self):
        """Test MCP uppercase mapping."""
        assert AGENT_FUNCTION_FIELD_MAPPINGS["is_mcp_server"] == "isMCPServer"


class TestIntegration:
    """Integration tests for full conversion flow."""

    def test_agent_update_kwargs_full(self):
        """Test full agent update kwargs conversion."""
        kwargs = {
            "json_output_structure": {"type": "object"},
            "enable_json_output": True,
            "has_ner": True,
            "parent_agent_id": "agent-123",
            "llm_url": "https://api.example.com",
            "unknown_field": "value",
        }
        result = convert_kwargs_to_api(kwargs, AGENT_FIELD_MAPPINGS, AGENT_EXPLICIT_PARAMS)
        
        assert result["jsonOutputStructure"] == {"type": "object"}
        assert result["enableJsonOutput"] is True
        assert result["hasNER"] is True
        assert result["parentAgentID"] == "agent-123"
        assert result["llmUrl"] == "https://api.example.com"
        assert result["unknownField"] == "value"

    def test_agent_function_update_kwargs_full(self):
        """Test full agent function update kwargs conversion."""
        kwargs = {
            "name": "test_function",
            "request_type": "post",
            "authorisation_id": "auth-123",
            "is_mcp_server": True,
            "oauth_connection_id": "oauth-123",
            "custom_execution_code": "print('hello')",
        }
        result = convert_kwargs_to_api(kwargs, AGENT_FUNCTION_FIELD_MAPPINGS, AGENT_FUNCTION_EXPLICIT_PARAMS)
        
        assert result["name"] == "test_function"
        assert result["requestType"] == "post"
        assert result["authorisationID"] == "auth-123"
        assert result["isMCPServer"] is True
        assert result["OAuthConnectionID"] == "oauth-123"
        assert result["customExecutionCode"] == "print('hello')"

    def test_user_can_bypass_with_camelcase(self):
        """Test user can bypass conversion by using camelCase directly."""
        kwargs = {
            "jsonOutputStructure": {"type": "object"},
            "hasNER": True,
        }
        result = convert_kwargs_to_api(kwargs, AGENT_FIELD_MAPPINGS)
        
        assert result["jsonOutputStructure"] == {"type": "object"}
        assert result["hasNER"] is True


"""Field mappings for AgentFunction type.

Maps SDK snake_case → GraphQL/API camelCase with exceptions.
Only exception cases are listed here - standard conversions use to_camel_case().

Derived from: /app/app/amplify/backend/api/toothfairyai/schema.graphql
"""

AGENT_FUNCTION_FIELD_MAPPINGS = {
    "workspace_id": "workspaceid",
    
    "authorisation_id": "authorisationID",
    "code_execution_environment_id": "codeExecutionEnvironmentID",
    "db_connection_id": "dbConnectionID",
    "pre_execution_fn_id": "preExecutionFnID",
    
    "oauth_connection_id": "OAuthConnectionID",
    
    "is_mcp_server": "isMCPServer",
}

AGENT_FUNCTION_EXPLICIT_PARAMS = set()


"""Shared utilities for field name conversion."""

from typing import Dict, Any, Set


def to_camel_case(key: str) -> str:
    """Standard snake_case → camelCase conversion.
    
    Does NOT handle exceptions - use field_mappings for that.
    Keys without underscores pass through unchanged.
    
    Args:
        key: snake_case field name
        
    Returns:
        camelCase field name
    """
    if '_' not in key:
        return key
    parts = key.split('_')
    return parts[0] + ''.join(word.capitalize() for word in parts[1:])


def convert_kwargs_to_api(
    kwargs: Dict[str, Any], 
    field_mappings: Dict[str, str],
    explicit_params: Set[str] = None
) -> Dict[str, Any]:
    """Convert SDK kwargs to API field names.
    
    Args:
        kwargs: User-provided keyword arguments (snake_case keys)
        field_mappings: Per-manager field mapping dictionary for exceptions
        explicit_params: Set of params already handled explicitly (skip these)
    
    Returns:
        Dictionary with API field names (camelCase keys)
    
    Example:
        >>> convert_kwargs_to_api(
        ...     {"json_output_structure": {}, "has_ner": True},
        ...     AGENT_FIELD_MAPPINGS,
        ...     {"label", "description"}
        ... )
        {"jsonOutputStructure": {}, "hasNER": True}
    """
    explicit_params = explicit_params or set()
    result = {}
    
    for key, value in kwargs.items():
        if key in explicit_params:
            continue
            
        if key in field_mappings:
            api_key = field_mappings[key]
        else:
            api_key = to_camel_case(key)
        
        result[api_key] = value
    
    return result


"""Tests for type definitions."""

import pytest

from toothfairyai.types import (
    Agent,
    AgentFunction,
    AgentResponse,
    Chat,
    CustomCodeExecutionEnvironment,
    CustomExecutionEnvironment,
    DatabaseConnection,
    Document,
    Entity,
    FileUploadResult,
    Folder,
    FolderTreeNode,
    Hosting,
    Message,
    Prompt,
    ScheduledJob,
    CronSchedule,
    CronExecutionConfig,
    CronNotificationConfig,
    CronRetryConfig,
    get_content_type,
)


class TestChatTypes:
    """Tests for chat-related types."""

    def test_chat_from_dict(self):
        """Test Chat.from_dict conversion."""
        data = {
            "id": "chat-123",
            "name": "Test Chat",
            "primaryRole": "user",
            "customerId": "customer-456",
            "customerInfo": {"name": "John"},
            "isAIReplying": True,
            "channelSettings": {"channel": "web", "provider": "default"},
        }
        chat = Chat.from_dict(data)

        assert chat.id == "chat-123"
        assert chat.name == "Test Chat"
        assert chat.primary_role == "user"
        assert chat.customer_id == "customer-456"
        assert chat.customer_info == {"name": "John"}
        assert chat.is_ai_replying is True
        assert chat.channel_settings is not None
        assert chat.channel_settings.channel == "web"

    def test_chat_from_dict_with_missing_fields(self):
        """Test Chat.from_dict with missing fields."""
        data = {"id": "chat-123"}
        chat = Chat.from_dict(data)

        assert chat.id == "chat-123"
        assert chat.name == ""
        assert chat.customer_info == {}
        assert chat.channel_settings is None

    def test_message_from_dict(self):
        """Test Message.from_dict conversion."""
        data = {
            "id": "msg-123",
            "chatID": "chat-456",
            "text": "Hello world",
            "role": "user",
            "userID": "user-789",
            "images": ["image1.jpg"],
        }
        message = Message.from_dict(data)

        assert message.id == "msg-123"
        assert message.chat_id == "chat-456"
        assert message.text == "Hello world"
        assert message.role == "user"
        assert message.images == ["image1.jpg"]


class TestDocumentTypes:
    """Tests for document-related types."""

    def test_document_from_dict(self):
        """Test Document.from_dict conversion."""
        data = {
            "id": "doc-123",
            "workspaceid": "ws-456",
            "userid": "user-789",
            "type": "readComprehensionPdf",
            "title": "Test Document",
            "topics": ["topic1", "topic2"],
            "folderid": "folder-123",
            "status": "published",
        }
        doc = Document.from_dict(data)

        assert doc.id == "doc-123"
        assert doc.workspace_id == "ws-456"
        assert doc.doc_type == "readComprehensionPdf"
        assert doc.title == "Test Document"
        assert doc.topics == ["topic1", "topic2"]

    def test_file_upload_result_from_dict(self):
        """Test FileUploadResult.from_dict conversion."""
        data = {
            "success": True,
            "originalFilename": "test.pdf",
            "sanitizedFilename": "test.pdf",
            "filename": "uuid-test.pdf",
            "importType": "file",
            "contentType": "application/pdf",
            "size": 1024,
            "sizeInMB": 0.001,
        }
        result = FileUploadResult.from_dict(data)

        assert result.success is True
        assert result.original_filename == "test.pdf"
        assert result.content_type == "application/pdf"
        assert result.size == 1024


class TestEntityTypes:
    """Tests for entity-related types."""

    def test_entity_from_dict(self):
        """Test Entity.from_dict conversion."""
        data = {
            "id": "entity-123",
            "workspaceid": "ws-456",
            "createdBy": "user-789",
            "label": "Dental Cleaning",
            "type": "topic",
            "description": "Professional cleaning",
            "emoji": "🦷",
        }
        entity = Entity.from_dict(data)

        assert entity.id == "entity-123"
        assert entity.label == "Dental Cleaning"
        assert entity.entity_type == "topic"
        assert entity.emoji == "🦷"


class TestFolderTypes:
    """Tests for folder-related types."""

    def test_folder_from_dict(self):
        """Test Folder.from_dict conversion."""
        data = {
            "id": "folder-123",
            "workspaceID": "ws-456",
            "createdBy": "user-789",
            "name": "Documents",
            "description": "Main documents folder",
            "status": "active",
            "parent": None,
        }
        folder = Folder.from_dict(data)

        assert folder.id == "folder-123"
        assert folder.name == "Documents"
        assert folder.status == "active"
        assert folder.parent is None

    def test_folder_tree_node_from_folder(self):
        """Test FolderTreeNode.from_folder conversion."""
        folder = Folder(
            id="folder-123",
            workspace_id="ws-456",
            created_by="user-789",
            name="Parent",
        )
        children = []
        node = FolderTreeNode.from_folder(folder, children)

        assert node.id == "folder-123"
        assert node.name == "Parent"
        assert node.children == []


class TestPromptTypes:
    """Tests for prompt-related types."""

    def test_prompt_from_dict(self):
        """Test Prompt.from_dict conversion."""
        data = {
            "id": "prompt-123",
            "workspaceID": "ws-456",
            "createdBy": "user-789",
            "type": "greeting",
            "label": "Welcome Prompt",
            "promptLength": 50,
            "interpolationString": "Hello {{name}}!",
            "availableToAgents": ["agent-1", "agent-2"],
        }
        prompt = Prompt.from_dict(data)

        assert prompt.id == "prompt-123"
        assert prompt.label == "Welcome Prompt"
        assert prompt.prompt_type == "greeting"
        assert prompt.interpolation_string == "Hello {{name}}!"
        assert prompt.available_to_agents == ["agent-1", "agent-2"]


class TestUtilityFunctions:
    """Tests for utility functions."""

    def test_get_content_type_pdf(self):
        """Test content type detection for PDF."""
        assert get_content_type("document.pdf") == "application/pdf"

    def test_get_content_type_image(self):
        """Test content type detection for images."""
        assert get_content_type("image.jpg") == "image/jpeg"
        assert get_content_type("image.jpeg") == "image/jpeg"
        assert get_content_type("image.png") == "image/png"

    def test_get_content_type_unknown(self):
        """Test content type detection for unknown extension."""
        assert get_content_type("file.xyz") == "application/octet-stream"

    def test_get_content_type_case_insensitive(self):
        """Test that content type detection is case insensitive."""
        assert get_content_type("document.PDF") == "application/pdf"
        assert get_content_type("image.JPG") == "image/jpeg"


class TestAgentTypes:
    """Tests for Agent type with all fields."""

    def test_agent_from_dict_basic(self):
        """Test Agent.from_dict with basic fields."""
        data = {
            "id": "agent-123",
            "workspaceID": "ws-456",
            "label": "Test Agent",
            "description": "A test agent",
            "goals": "Help users",
        }
        agent = Agent.from_dict(data)

        assert agent.id == "agent-123"
        assert agent.workspace_id == "ws-456"
        assert agent.label == "Test Agent"
        assert agent.description == "A test agent"
        assert agent.goals == "Help users"

    def test_agent_from_dict_with_llm_settings(self):
        """Test Agent.from_dict with LLM settings."""
        data = {
            "id": "agent-123",
            "workspaceID": "ws-456",
            "label": "Test Agent",
            "llmEndpoint": "https://api.openai.com",
            "llmUrl": "https://api.openai.com/v1",
            "llmUrlStatus": "ready",
            "llmUrlType": "serverless",
            "llmBaseModel": "gpt-4",
            "llmProvider": "openai",
            "llmTopK": 40.0,
            "llmSeed": 42.0,
        }
        agent = Agent.from_dict(data)

        assert agent.llm_endpoint == "https://api.openai.com"
        assert agent.llm_url == "https://api.openai.com/v1"
        assert agent.llm_url_status == "ready"
        assert agent.llm_url_type == "serverless"
        assert agent.llm_base_model == "gpt-4"
        assert agent.llm_provider == "openai"
        assert agent.llm_top_k == 40.0
        assert agent.llm_seed == 42.0

    def test_agent_from_dict_with_reasoning_settings(self):
        """Test Agent.from_dict with reasoning settings."""
        data = {
            "id": "agent-123",
            "workspaceID": "ws-456",
            "label": "Test Agent",
            "agentType": "Chatbot",
            "reasoningMode": "auto",
            "reasoningEffort": "high",
            "fnReasoningEffort": "medium",
            "reasoningBudget": 1000,
        }
        agent = Agent.from_dict(data)

        assert agent.agent_type == "Chatbot"
        assert agent.reasoning_mode == "auto"
        assert agent.reasoning_effort == "high"
        assert agent.fn_reasoning_effort == "medium"
        assert agent.reasoning_budget == 1000

    def test_agent_from_dict_with_voice_settings(self):
        """Test Agent.from_dict with voice settings."""
        data = {
            "id": "agent-123",
            "workspaceID": "ws-456",
            "label": "Test Agent",
            "voiceName": "alloy",
            "sttModel": "whisper-1",
            "ttsModel": "tts-1",
            "llmVoiceModel": "gpt-4-audio",
            "defaultVoiceLanguage": "en-US",
            "voiceMinSilenceDuration": 0.5,
            "voiceMinEndpointingDelay": 1.0,
            "voiceMinInterruptionDuration": 0.3,
            "enableVoiceIntermissions": True,
            "voiceBackgroundNoise": "none",
            "autoVoiceMode": True,
            "voiceSensitivity": "high",
        }
        agent = Agent.from_dict(data)

        assert agent.voice_name == "alloy"
        assert agent.stt_model == "whisper-1"
        assert agent.tts_model == "tts-1"
        assert agent.llm_voice_model == "gpt-4-audio"
        assert agent.default_voice_language == "en-US"
        assert agent.voice_min_silence_duration == 0.5
        assert agent.voice_min_endpointing_delay == 1.0
        assert agent.voice_min_interruption_duration == 0.3
        assert agent.enable_voice_intermissions is True
        assert agent.auto_voice_mode is True

    def test_agent_from_dict_with_planning_settings(self):
        """Test Agent.from_dict with planning settings."""
        data = {
            "id": "agent-123",
            "workspaceID": "ws-456",
            "label": "Test Agent",
            "agentsPool": ["agent-1", "agent-2"],
            "mcps": ["mcp-1"],
            "maxPlanningSteps": 10,
            "maxBrowserSteps": 5,
            "planningInstructions": "Plan carefully",
            "plannerExecutionAttempts": 3,
            "plannerRequiresApprovalForPlanExecution": True,
            "plannerEmailOnApprovalRequest": True,
            "plannerApproverEmails": ["admin@example.com"],
            "allowReplanning": True,
        }
        agent = Agent.from_dict(data)

        assert agent.agents_pool == ["agent-1", "agent-2"]
        assert agent.mcps == ["mcp-1"]
        assert agent.max_planning_steps == 10
        assert agent.max_browser_steps == 5
        assert agent.planning_instructions == "Plan carefully"
        assert agent.planner_requires_approval_for_plan_execution is True
        assert agent.planner_approver_emails == ["admin@example.com"]

    def test_agent_from_dict_with_upload_generation_settings(self):
        """Test Agent.from_dict with upload/generation settings."""
        data = {
            "id": "agent-123",
            "workspaceID": "ws-456",
            "label": "Test Agent",
            "allowImagesUpload": True,
            "allowVideosUpload": False,
            "allowAudiosUpload": True,
            "allowDocsUpload": True,
            "allowImagesGeneration": True,
            "baseImageGenerationModel": "dall-e-3",
            "maxImagesGenerated": 4,
            "advancedImageGeneration": True,
            "allowVideosGeneration": False,
            "allow3dModelGeneration": False,
            "allowIntrospection": True,
        }
        agent = Agent.from_dict(data)

        assert agent.allow_images_upload is True
        assert agent.allow_videos_upload is False
        assert agent.allow_audios_upload is True
        assert agent.allow_docs_upload is True
        assert agent.allow_images_generation is True
        assert agent.base_image_generation_model == "dall-e-3"
        assert agent.max_images_generated == 4
        assert agent.advanced_image_generation is True
        assert agent.allow_introspection is True

    def test_agent_from_dict_with_sip_settings(self):
        """Test Agent.from_dict with SIP settings."""
        data = {
            "id": "agent-123",
            "workspaceID": "ws-456",
            "label": "Test Agent",
            "sipPhoneNumber": "+1234567890",
            "sipDispatchRuleId": "rule-123",
            "enableInboundSip": True,
            "enableOutboundSip": False,
        }
        agent = Agent.from_dict(data)

        assert agent.sip_phone_number == "+1234567890"
        assert agent.sip_dispatch_rule_id == "rule-123"
        assert agent.enable_inbound_sip is True
        assert agent.enable_outbound_sip is False

    def test_agent_from_dict_with_rag_settings(self):
        """Test Agent.from_dict with RAG settings."""
        data = {
            "id": "agent-123",
            "workspaceID": "ws-456",
            "label": "Test Agent",
            "agenticRAG": True,
            "reRank": True,
            "ragSearchStrategy": "parallel",
            "minChunksBeforeCompleteness": 0.5,
            "ragCompletenessThreshold": 0.8,
            "ragInstructions": "Search thoroughly",
            "keywordsWeightInRetrieval": 0.3,
        }
        agent = Agent.from_dict(data)

        assert agent.agentic_rag is True
        assert agent.re_rank is True
        assert agent.rag_search_strategy == "parallel"
        assert agent.min_chunks_before_completeness == 0.5
        assert agent.rag_completeness_threshold == 0.8
        assert agent.rag_instructions == "Search thoroughly"


class TestAgentFunctionTypes:
    """Tests for AgentFunction type with all fields."""

    def test_agent_function_from_dict_basic(self):
        """Test AgentFunction.from_dict with basic fields."""
        data = {
            "id": "fn-123",
            "name": "Test Function",
            "description": "A test function",
            "workspaceID": "ws-456",
            "url": "https://api.example.com/execute",
            "requestType": "post",
            "authorisationType": "bearer",
        }
        fn = AgentFunction.from_dict(data)

        assert fn.id == "fn-123"
        assert fn.name == "Test Function"
        assert fn.description == "A test function"
        assert fn.url == "https://api.example.com/execute"
        assert fn.request_type == "post"
        assert fn.authorisation_type == "bearer"

    def test_agent_function_from_dict_with_skill_settings(self):
        """Test AgentFunction.from_dict with agent skill settings."""
        data = {
            "id": "fn-123",
            "name": "Skill Function",
            "description": "An agent skill",
            "isAgentSkill": True,
            "agentSkillScript": "def execute(): return 'result'",
            "codeExecutionEnvironmentID": "env-123",
        }
        fn = AgentFunction.from_dict(data)

        assert fn.is_agent_skill is True
        assert fn.agent_skill_script == "def execute(): return 'result'"
        assert fn.code_execution_environment_id == "env-123"

    def test_agent_function_from_dict_with_handoff_settings(self):
        """Test AgentFunction.from_dict with handoff settings."""
        data = {
            "id": "fn-123",
            "name": "Handoff Function",
            "description": "A handoff function",
            "isHandOffFn": True,
            "handOffAgent": "agent-456",
            "handOffToOriginalAgentOnCompletion": True,
            "handOffToOriginalAgentOnFailure": False,
            "handOffAgentOnCompletion": "agent-789",
            "handOffAgentOnFailure": "agent-error",
            "usersToHandOffTo": ["user-1", "user-2"],
            "handedOffToHumanOnCall": True,
        }
        fn = AgentFunction.from_dict(data)

        assert fn.is_hand_off_fn is True
        assert fn.hand_off_agent == "agent-456"
        assert fn.hand_off_to_original_agent_on_completion is True
        assert fn.hand_off_to_original_agent_on_failure is False
        assert fn.hand_off_agent_on_completion == "agent-789"
        assert fn.users_to_hand_off_to == ["user-1", "user-2"]
        assert fn.handed_off_to_human_on_call is True

    def test_agent_function_from_dict_with_chat_settings(self):
        """Test AgentFunction.from_dict with chat settings."""
        data = {
            "id": "fn-123",
            "name": "Chat Function",
            "description": "A chat function",
            "isChatFn": True,
            "chatScript": "async def chat(message): return 'response'",
            "chatAction": "suggestion",
        }
        fn = AgentFunction.from_dict(data)

        assert fn.is_chat_fn is True
        assert fn.chat_script == "async def chat(message): return 'response'"
        assert fn.chat_action == "suggestion"

    def test_agent_function_from_dict_with_html_settings(self):
        """Test AgentFunction.from_dict with HTML settings."""
        data = {
            "id": "fn-123",
            "name": "HTML Function",
            "description": "An HTML function",
            "isHtmlFn": True,
            "htmlUrl": "https://example.com/form",
            "htmlScript": "<form>...</form>",
            "isHtmlRedirectFn": False,
            "htmlFormOnSuccessfulSubmitMessage": "Success!",
            "htmlFormOnFailedSubmitMessage": "Error!",
        }
        fn = AgentFunction.from_dict(data)

        assert fn.is_html_fn is True
        assert fn.html_url == "https://example.com/form"
        assert fn.html_script == "<form>...</form>"
        assert fn.is_html_redirect_fn is False
        assert fn.html_form_on_successful_submit_message == "Success!"

    def test_agent_function_from_dict_with_database_settings(self):
        """Test AgentFunction.from_dict with database settings."""
        data = {
            "id": "fn-123",
            "name": "Database Function",
            "description": "A database function",
            "isDatabaseScript": True,
            "dbScript": "SELECT * FROM users",
            "dbConnectionID": "db-123",
            "dbSchemaMetadata": {"table": "users"},
            "preExecutionScript": "SET search_path = public",
            "postExecutionScript": "COMMIT",
        }
        fn = AgentFunction.from_dict(data)

        assert fn.is_database_script is True
        assert fn.db_script == "SELECT * FROM users"
        assert fn.db_connection_id == "db-123"
        assert fn.db_schema_metadata == {"table": "users"}
        assert fn.pre_execution_script == "SET search_path = public"

    def test_agent_function_from_dict_with_mcp_settings(self):
        """Test AgentFunction.from_dict with MCP settings."""
        data = {
            "id": "fn-123",
            "name": "MCP Server",
            "description": "An MCP server",
            "isMCPServer": True,
            "scope": "customer",
            "jsonPathExtractor": "$.data",
            "dependsOnBeforeExecution": ["fn-1"],
            "dependsOnAfterExecution": ["fn-2"],
        }
        fn = AgentFunction.from_dict(data)

        assert fn.is_mcp_server is True
        assert fn.scope == "customer"
        assert fn.json_path_extractor == "$.data"
        assert fn.depends_on_before_execution == ["fn-1"]
        assert fn.depends_on_after_execution == ["fn-2"]

    def test_agent_function_from_dict_with_graphql_settings(self):
        """Test AgentFunction.from_dict with GraphQL settings."""
        data = {
            "id": "fn-123",
            "name": "GraphQL Function",
            "description": "A GraphQL function",
            "requestType": "graphql_query",
            "graphqlQuery": "{ users { id name } }",
            "graphqlMutation": None,
            "graphqlOperationName": "GetUsers",
        }
        fn = AgentFunction.from_dict(data)

        assert fn.request_type == "graphql_query"
        assert fn.graphql_query == "{ users { id name } }"
        assert fn.graphql_operation_name == "GetUsers"


class TestCustomCodeExecutionEnvironmentTypes:
    """Tests for CustomCodeExecutionEnvironment (Hook) type."""

    def test_custom_code_execution_environment_from_dict(self):
        """Test CustomCodeExecutionEnvironment.from_dict."""
        data = {
            "id": "hook-123",
            "name": "Test Hook",
            "workspaceID": "ws-456",
            "description": "A test hook",
            "functionName": "execute",
            "customExecutionCode": "def execute(): pass",
            "customExecutionInstructions": "Run this code",
            "customExecutionSecrets": {"api_key": "secret"},
            "staticDocs": {"doc1": "content"},
            "availableLibraries": "numpy, pandas",
            "allowExternalAPI": True,
            "hardcodedScript": False,
            "isTemplate": False,
            "createdBy": "user-123",
            "updatedBy": "user-456",
        }
        hook = CustomCodeExecutionEnvironment.from_dict(data)

        assert hook.id == "hook-123"
        assert hook.name == "Test Hook"
        assert hook.workspace_id == "ws-456"
        assert hook.function_name == "execute"
        assert hook.custom_execution_code == "def execute(): pass"
        assert hook.custom_execution_secrets == {"api_key": "secret"}
        assert hook.static_docs == {"doc1": "content"}
        assert hook.allow_external_api is True
        assert hook.is_template is False
        assert hook.created_by == "user-123"
        assert hook.updated_by == "user-456"


class TestCustomExecutionEnvironmentTypes:
    """Tests for CustomExecutionEnvironment type."""

    def test_custom_execution_environment_from_dict(self):
        """Test CustomExecutionEnvironment.from_dict."""
        data = {
            "id": "env-123",
            "name": "Remote Environment",
            "workspaceID": "ws-456",
            "description": "A remote execution environment",
            "functionName": "run_remote",
            "availableLibraries": "requests, httpx",
            "ip": "192.168.1.100",
            "port": "8080",
            "metadata": {"region": "us-east-1"},
            "isGlobal": False,
            "createdBy": "user-123",
        }
        env = CustomExecutionEnvironment.from_dict(data)

        assert env.id == "env-123"
        assert env.name == "Remote Environment"
        assert env.ip == "192.168.1.100"
        assert env.port == "8080"
        assert env.metadata == {"region": "us-east-1"}
        assert env.is_global is False


class TestScheduledJobTypes:
    """Tests for ScheduledJob (CronJob) type with nested types."""

    def test_cron_schedule_from_dict(self):
        """Test CronSchedule.from_dict."""
        data = {
            "frequency": "DAILY",
            "interval": 1,
            "cronExpression": None,
            "dayOfWeek": None,
            "dayOfMonth": None,
            "hour": 9,
            "minute": 30,
            "startDate": "2024-01-01T00:00:00Z",
            "endDate": "2024-12-31T23:59:59Z",
        }
        schedule = CronSchedule.from_dict(data)

        assert schedule.frequency == "DAILY"
        assert schedule.interval == 1
        assert schedule.hour == 9
        assert schedule.minute == 30
        assert schedule.start_date == "2024-01-01T00:00:00Z"

    def test_cron_execution_config_from_dict(self):
        """Test CronExecutionConfig.from_dict."""
        data = {
            "timeout": 300,
            "maxDuration": 600,
            "priority": "HIGH",
            "inputParameters": {"key": "value"},
            "outputFormat": "json",
            "webhookUrl": "https://example.com/webhook",
            "webhookHeaders": {"Authorization": "Bearer token"},
        }
        config = CronExecutionConfig.from_dict(data)

        assert config.timeout == 300
        assert config.max_duration == 600
        assert config.priority == "HIGH"
        assert config.input_parameters == {"key": "value"}
        assert config.webhook_url == "https://example.com/webhook"

    def test_cron_notification_config_from_dict(self):
        """Test CronNotificationConfig.from_dict."""
        data = {
            "onSuccess": True,
            "onFailure": True,
            "onStart": False,
            "emailRecipients": ["admin@example.com", "ops@example.com"],
            "communicationServiceID": "svc-123",
        }
        config = CronNotificationConfig.from_dict(data)

        assert config.on_success is True
        assert config.on_failure is True
        assert config.on_start is False
        assert config.email_recipients == ["admin@example.com", "ops@example.com"]
        assert config.communication_service_id == "svc-123"

    def test_cron_retry_config_from_dict(self):
        """Test CronRetryConfig.from_dict."""
        data = {
            "maxRetries": 5,
            "retryInterval": 120,
            "backoffMultiplier": 2.0,
            "maxRetryInterval": 3600,
        }
        config = CronRetryConfig.from_dict(data)

        assert config.max_retries == 5
        assert config.retry_interval == 120
        assert config.backoff_multiplier == 2.0
        assert config.max_retry_interval == 3600

    def test_scheduled_job_from_dict_with_nested_types(self):
        """Test ScheduledJob.from_dict with nested configuration types."""
        data = {
            "id": "job-123",
            "name": "Daily Report",
            "workspaceID": "ws-456",
            "description": "Generate daily report",
            "agentID": "agent-789",
            "customPromptID": "prompt-123",
            "forcedPrompt": "Generate report for {{date}}",
            "timezone": "America/New_York",
            "isActive": True,
            "lastRunAt": "2024-01-01T09:00:00Z",
            "nextRunAt": "2024-01-02T09:00:00Z",
            "status": "ACTIVE",
            "schedule": {
                "frequency": "DAILY",
                "hour": 9,
                "minute": 0,
            },
            "executionConfig": {
                "timeout": 300,
                "priority": "MEDIUM",
            },
            "notificationConfig": {
                "onSuccess": True,
                "emailRecipients": ["admin@example.com"],
            },
            "retryConfig": {
                "maxRetries": 3,
            },
            "metadata": {"tags": ["daily", "report"]},
            "createdBy": "user-123",
            "updatedBy": "user-456",
            "createdAt": "2024-01-01T00:00:00Z",
            "updatedAt": "2024-01-01T12:00:00Z",
        }
        job = ScheduledJob.from_dict(data)

        assert job.id == "job-123"
        assert job.name == "Daily Report"
        assert job.workspace_id == "ws-456"
        assert job.agent_id == "agent-789"
        assert job.custom_prompt_id == "prompt-123"
        assert job.timezone == "America/New_York"
        assert job.is_active is True
        assert job.status == "ACTIVE"
        assert job.schedule is not None
        assert job.schedule.frequency == "DAILY"
        assert job.schedule.hour == 9
        assert job.execution_config is not None
        assert job.execution_config.timeout == 300
        assert job.notification_config is not None
        assert job.notification_config.on_success is True
        assert job.retry_config is not None
        assert job.retry_config.max_retries == 3
        assert job.metadata == {"tags": ["daily", "report"]}

    def test_scheduled_job_from_dict_minimal(self):
        """Test ScheduledJob.from_dict with minimal data."""
        data = {
            "id": "job-123",
            "name": "Simple Job",
        }
        job = ScheduledJob.from_dict(data)

        assert job.id == "job-123"
        assert job.name == "Simple Job"
        assert job.timezone == "UTC"
        assert job.status == "PENDING"
        assert job.schedule is None
        assert job.execution_config is None
        assert job.notification_config is None


class TestHostingTypes:
    """Tests for Hosting (Connection) type."""

    def test_hosting_from_dict(self):
        """Test Hosting.from_dict."""
        data = {
            "id": "hosting-123",
            "name": "OpenAI Connection",
            "type": "openai",
            "workspaceID": "ws-456",
            "description": "OpenAI API connection",
            "tokenSecret": "openai-api-key",
            "endpoint": "https://api.openai.com",
            "modelName": "gpt-4",
            "baseModel": "gpt-4",
            "customModel": False,
            "apiVersion": "v1",
            "createdBy": "user-123",
            "updatedBy": "user-456",
        }
        hosting = Hosting.from_dict(data)

        assert hosting.id == "hosting-123"
        assert hosting.name == "OpenAI Connection"
        assert hosting.type == "openai"
        assert hosting.endpoint == "https://api.openai.com"
        assert hosting.model_name == "gpt-4"
        assert hosting.custom_model is False
        assert hosting.api_version == "v1"


class TestDatabaseConnectionTypes:
    """Tests for DatabaseConnection type."""

    def test_database_connection_from_dict(self):
        """Test DatabaseConnection.from_dict."""
        data = {
            "id": "db-123",
            "name": "Postgres Database",
            "type": "postgresql",
            "host": "db.example.com",
            "port": "5432",
            "username": "admin",
            "passwordSecret": "db-password",
            "database": "production",
            "workspaceID": "ws-456",
            "description": "Production database",
            "schema": "public",
            "ssl": True,
            "ssh": False,
            "createdBy": "user-123",
        }
        db = DatabaseConnection.from_dict(data)

        assert db.id == "db-123"
        assert db.name == "Postgres Database"
        assert db.type == "postgresql"
        assert db.host == "db.example.com"
        assert db.port == "5432"
        assert db.username == "admin"
        assert db.password_secret == "db-password"
        assert db.database == "production"
        assert db.ssl is True
        assert db.ssh is False


class TestPromptTypesExtended:
    """Extended tests for Prompt type."""

    def test_prompt_from_dict_with_all_fields(self):
        """Test Prompt.from_dict with all fields."""
        data = {
            "id": "prompt-123",
            "workspaceID": "ws-456",
            "type": "greeting",
            "label": "Welcome Prompt",
            "promptLength": 150,
            "interpolationString": "Hello {{name}}! Welcome to our service. We're here to help you.",
            "scope": "workspace",
            "style": "formal",
            "domain": "customerService",
            "promptPlaceholder": "Enter your name",
            "availableToAgents": ["agent-1", "agent-2"],
            "createdBy": "user-123",
            "updatedBy": "user-456",
            "createdAt": "2024-01-01T00:00:00Z",
            "updatedAt": "2024-01-02T00:00:00Z",
        }
        prompt = Prompt.from_dict(data)

        assert prompt.id == "prompt-123"
        assert prompt.prompt_type == "greeting"
        assert prompt.label == "Welcome Prompt"
        assert prompt.prompt_length == 150
        assert prompt.interpolation_string == "Hello {{name}}! Welcome to our service. We're here to help you."
        assert prompt.scope == "workspace"
        assert prompt.style == "formal"
        assert prompt.domain == "customerService"
        assert prompt.prompt_placeholder == "Enter your name"
        assert prompt.available_to_agents == ["agent-1", "agent-2"]
        assert prompt.created_by == "user-123"
        assert prompt.updated_by == "user-456"



"""Connection manager for handling connections (Hosting) operations."""

from typing import TYPE_CHECKING, Any, Dict, List, Optional

from ..types import Connection, Hosting, HostingType, ListResponse

if TYPE_CHECKING:
    from ..client import ToothFairyClient


class ConnectionManager:
    """Manager for connections (Hosting) operations.

    This manager provides methods to create, update, and manage connections (Hostings).
    Note: In the API, /connection/* endpoints map to Hosting entities.

    Example:
        >>> client = ToothFairyClient(api_key="...", workspace_id="...")
        >>> connection = client.connections.create(
        ...     name="OpenAI Connection",
        ...     type="openai",
        ...     endpoint="https://api.openai.com",
        ... )
    """

    def __init__(self, client: "ToothFairyClient"):
        """Initialize the ConnectionManager.

        Args:
            client: The ToothFairyClient instance.
        """
        self._client = client

    def create(
        self,
        name: str,
        type: HostingType,
        endpoint: Optional[str] = None,
        model_name: Optional[str] = None,
        base_model: Optional[str] = None,
        custom_model: bool = False,
        api_version: Optional[str] = None,
        token_secret: Optional[str] = None,
        description: Optional[str] = None,
    ) -> Hosting:
        """Create a new connection (Hosting).

        Args:
            name: Connection name.
            type: Hosting type (azure_openai, aws, local, google, anthropic, together, tf, replicate, azure, openai, groq, ollama).
            endpoint: API endpoint URL.
            model_name: Model name to use.
            base_model: Base model identifier.
            custom_model: Whether this is a custom model.
            api_version: API version (for Azure OpenAI).
            token_secret: Secret name for API token.
            description: Connection description.

        Returns:
            The created Hosting object.
        """
        data: Dict[str, Any] = {
            "name": name,
            "type": type,
            "customModel": custom_model,
        }

        if endpoint is not None:
            data["endpoint"] = endpoint
        if model_name is not None:
            data["modelName"] = model_name
        if base_model is not None:
            data["baseModel"] = base_model
        if api_version is not None:
            data["apiVersion"] = api_version
        if token_secret is not None:
            data["tokenSecret"] = token_secret
        if description is not None:
            data["description"] = description

        response = self._client.request("POST", "/connection/create", data=data)
        return Hosting.from_dict(response)

    def update(
        self,
        connection_id: str,
        **kwargs: Any,
    ) -> Hosting:
        """Update a connection (Hosting).

        Args:
            connection_id: ID of the connection to update.
            **kwargs: Fields to update (name, type, endpoint, modelName, baseModel, customModel, apiVersion, tokenSecret, description).

        Returns:
            The updated Hosting object.
        """
        data: Dict[str, Any] = {"id": connection_id}
        data.update(kwargs)
        response = self._client.request("POST", "/connection/update", data=data)
        return Hosting.from_dict(response)

    def delete(self, connection_id: str) -> Dict[str, bool]:
        """Delete a connection.

        Args:
            connection_id: ID of the connection to delete.

        Returns:
            A dictionary with success status.
        """
        self._client.request("DELETE", f"/connection/delete/{connection_id}")
        return {"success": True}

    def get(self, connection_id: str) -> Hosting:
        """Get a connection by ID.

        Args:
            connection_id: ID of the connection to retrieve.

        Returns:
            The Hosting object.
        """
        response = self._client.request("GET", f"/connection/get/{connection_id}")
        return Hosting.from_dict(response)

    def list(
        self,
        limit: Optional[int] = None,
        offset: Optional[int] = None,
    ) -> ListResponse:
        """List all connections.

        Args:
            limit: Maximum number of connections to return.
            offset: Number of connections to skip.

        Returns:
            A ListResponse containing the connections.
        """
        params: Dict[str, Any] = {}
        if limit is not None:
            params["limit"] = limit
        if offset is not None:
            params["offset"] = offset

        response = self._client.request("GET", "/connection/list", params=params)

        items = []
        if isinstance(response, list):
            items = [Hosting.from_dict(item) for item in response]
        elif isinstance(response, dict):
            items = [Hosting.from_dict(item) for item in response.get("items", [])]

        return ListResponse(items=items)

    def get_by_type(self, hosting_type: HostingType) -> List[Hosting]:
        """Get connections by hosting type.

        Args:
            hosting_type: Hosting type to filter by (openai, azure, anthropic, etc.).

        Returns:
            A list of Hosting objects with the specified type.
        """
        result = self.list()
        return [conn for conn in result.items if conn.type == hosting_type]

    def search(self, search_term: str) -> List[Hosting]:
        """Search connections by name.

        Args:
            search_term: Term to search for in connection names.

        Returns:
            A list of matching Hosting objects.
        """
        all_connections = self.list()
        search_lower = search_term.lower()
        return [conn for conn in all_connections.items if search_lower in conn.name.lower()]



"""Type definitions for the ToothFairyAI SDK."""

from dataclasses import dataclass, field
from typing import Any, Callable, Dict, List, Literal, Optional, TypedDict, Union


def _get_workspace_id(data: Dict[str, Any], default: str = "") -> str:
    """Get workspace_id from data, handling both camelCase and lowercase keys.

    Args:
        data: Dictionary containing workspace ID.
        default: Default value if not found.

    Returns:
        Workspace ID string.
    """
    return (
        data.get("workspaceID")
        or data.get("workspaceid")
        or data.get("workspace_id")
        or default
    )


# Type aliases.
EntityType = Literal["intent", "ner", "topic"]
EntityAIType = Literal["ner", "intent", "topic"]
DocumentType = Literal["readComprehensionUrl", "readComprehensionPdf", "readComprehensionFile"]
MessageRole = Literal["user", "assistant", "system"]
AgentMode = Literal["retriever", "coder", "chatter", "planner", "computer", "voice", "accuracy"]

# Enum types from GraphQL schema
AgentType = Literal["Chatbot", "BusinessAnalyst", "ContentCreator", "Default"]
ReasoningMode = Literal["auto", "dynamic", "always", "never"]
ReasoningEffort = Literal["none", "low", "medium", "high"]
PlanType = Literal["base", "pro", "business", "enterprise"]
Department = Literal[
    "HUMAN_RESOURCES",
    "FINANCE_AND_ACCOUNTING",
    "SALES",
    "MARKETING",
    "OPERATIONS",
    "INFORMATION_TECHNOLOGY",
    "CUSTOMER_SERVICE",
    "RESEARCH_AND_DEVELOPMENT",
    "LEGAL_AND_COMPLIANCE",
    "SUPPLY_CHAIN_MANAGEMENT",
    "PROJECT_MANAGEMENT",
    "GENERAL_PURPOSE",
]
QaUrlStatus = Literal["ready", "notReady", "dismissed", "unknown", "swapping"]
QaUrlType = Literal["serverless", "provisioned", "elastic"]
SearchStrategy = Literal["parallel", "targeted_first", "vector_first"]
InternetSearchMode = Literal["search", "news", "images", "videos", "maps", "places", "shopping"]
FunctionScope = Literal["customerAuthentication", "customer", "caseAuthentication", "case"]
AgentFunctionChatAction = Literal[
    "suggestion", "nextQuestion", "humanSupport", "redirection", "script"
]
ChatMode = Literal[
    "speed",
    "accuracy",
    "custom",
    "retriever",
    "coder",
    "mathematician",
    "chatter",
    "planner",
    "computer",
    "voice",
]
FunctionRequestType = Literal[
    "get", "post", "put", "delete", "patch", "custom", "graphql_query", "graphql_mutation"
]
AuthorisationType = Literal[
    "bearer",
    "apikey",
    "oauth",
    "password",
    "env",
    "generic",
    "none",
    "username_and_password",
    "basic",
]
Visibility = Literal["private", "public", "shared"]
WorkspaceAIType = Literal["hierarchical", "flat"]
BaseModelType = Literal["productivity", "sentiment", "none"]
TrainingStatus = Literal[
    "dispatched",
    "inProgress",
    "completed",
    "inError",
    "cancelled",
    "cancellationRequested",
    "training",
    "swapping",
    "swapModelFailed",
    "swapModelRequested",
    "datasetReady",
    "modelReady",
    "fineTuningRequested",
]
BatchJobStatus = Literal[
    "dispatched", "inProgress", "completed", "inError", "cancelled", "cancellationRequested"
]
RequestStatus = Literal[
    "success",
    "failed",
    "timeout",
    "fulfilled",
    "pending",
    "inProgress",
    "cancelled",
    "awaitingUserInput",
]
RequestType = Literal[
    "translator",
    "answerer",
    "parser",
    "tokenizer",
    "embedder",
    "detector",
    "questioner",
    "generator",
    "chatter",
    "planner",
    "analyser",
]
ModelType = Literal[
    "extraSmall", "small", "medium", "large", "extraLarge", "local", "custom", "external"
]
OutputType = Literal[
    "images",
    "videos",
    "modelling",
    "text",
    "code_interpreter",
    "canvas",
    "internet_search",
    "deep_thinking",
    "browser",
    "audio",
    "voice",
    "rag",
]
SiteStatus = Literal[
    "noDomain",
    "noAnswer",
    "active",
    "syncing",
    "inactive",
    "invalidToken",
    "failed",
    "pending",
    "inProgress",
    "validationTokenCreationError",
    "readyForValidation",
    "tokenRequested",
    "noSitemap",
]
FolderStatus = Literal["active", "inactive"]
ChatVisibility = Literal["public", "private", "widget", "job", "incognito"]
HumanFeedback = Literal["positive", "negative", "neutral"]
HumanFeedbackType = Literal[
    "multilanguage",
    "documentation",
    "reasoning",
    "images",
    "charting",
    "api",
    "code",
    "math",
    "moderation",
]
MessageType = Literal["system", "handoff", "notification"]
CommunicationChannel = Literal["email", "sms", "whatsapp", "linkedin", "sip"]
CommunicationProvider = Literal[
    "sendgrid",
    "ses",
    "mailgun",
    "postmark",
    "mailjet",
    "twilio",
    "whatsapp",
    "sms_magic",
    "sms_magic_sf",
    "linkedin",
]
HostingType = Literal[
    "azure_openai",
    "aws",
    "local",
    "google",
    "anthropic",
    "together",
    "tf",
    "replicate",
    "azure",
    "openai",
    "groq",
    "ollama",
]
CronJobStatus = Literal["ACTIVE", "PAUSED", "DISABLED", "RUNNING", "COMPLETED", "FAILED", "PENDING"]
CronJobFrequency = Literal["MINUTES", "HOURLY", "DAILY", "WEEKLY", "MONTHLY", "YEARLY", "CUSTOM"]
UserType = Literal["Root", "SuperAdmin", "ContentAdmin", "BillingAdmin", "AIEngineer", "User"]
SubscriptionStatus = Literal["active", "inactive"]
LogType = Literal["import", "export", "entityDeletion", "documentImport"]
LogStatus = Literal["dispatched", "inProgress", "completed", "inError"]
TrainingLogType = Literal[
    "nlpTraining", "readComprehensionTraining", "generativeTraining", "conversationalTraining"
]
BatchLogType = Literal["nlp", "rc", "gn", "bm", "abm"]
Priority = Literal["CRITICAL", "HIGH", "MEDIUM", "LOW", "NONE"]
VoucherStatus = Literal["ACTIVE", "INACTIVE"]
VoucherType = Literal["CREDIT", "FREE_UPGRADE"]
ChunkingStrategy = Literal["keywords", "summary"]
ExtractionStrategy = Literal["safe", "brute"]
GenerationMode = Literal["speed", "accuracy", "custom"]
GenerationDomains = Literal[
    "editorial",
    "customerService",
    "sales",
    "marketing",
    "hr",
    "finance",
    "legal",
    "medical",
    "science",
    "technology",
    "default",
]
GenerationStyles = Literal[
    "default",
    "short",
    "long",
    "funny",
    "informal",
    "formal",
    "interview",
    "academic",
    "business",
    "creative",
    "scientific",
    "poetry",
    "news",
]
GenerationScopes = Literal[
    "generation",
    "tweet",
    "post",
    "title",
    "email",
    "paraphrase",
    "paraphraseLiteral",
    "summary",
    "simplify",
    "custom",
]


# Configuration types
@dataclass
class ToothFairyClientConfig:
    """Configuration for the ToothFairyClient."""

    api_key: str
    workspace_id: str
    base_url: str = "https://api.toothfairyai.com"
    ai_url: str = "https://ai.toothfairyai.com"
    ai_stream_url: str = "https://ais.toothfairyai.com"
    timeout: int = 120  # seconds


# Chat types
@dataclass
class ChannelSettings:
    """Channel settings for a chat."""

    channel: Optional[str] = None
    provider: Optional[str] = None


@dataclass
class Chat:
    """Represents a chat conversation."""

    id: str
    name: str = ""
    primary_role: str = ""
    external_participant_id: str = ""
    channel_settings: Optional[ChannelSettings] = None
    customer_id: str = ""
    customer_info: Dict[str, Any] = field(default_factory=dict)
    is_ai_replying: bool = False
    created_at: Optional[str] = None
    updated_at: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Chat":
        """Create a Chat instance from a dictionary."""
        channel_settings = None
        if data.get("channelSettings"):
            channel_settings = ChannelSettings(
                channel=data["channelSettings"].get("channel"),
                provider=data["channelSettings"].get("provider"),
            )
        return cls(
            id=data.get("id", ""),
            name=data.get("name", ""),
            primary_role=data.get("primaryRole", ""),
            external_participant_id=data.get("externalParticipantId", ""),
            channel_settings=channel_settings,
            customer_id=data.get("customerId", ""),
            customer_info=data.get("customerInfo", {}),
            is_ai_replying=data.get("isAIReplying", False),
            created_at=data.get("createdAt"),
            updated_at=data.get("updatedAt"),
        )


@dataclass
class ChatCreateData:
    """Data for creating a chat."""

    name: str = ""
    customer_id: str = ""
    customer_info: Dict[str, Any] = field(default_factory=dict)
    primary_role: str = "user"
    external_participant_id: str = ""
    channel_settings: Optional[Dict[str, str]] = None


@dataclass
class Message:
    """Represents a chat message."""

    id: str
    chat_id: str
    text: str
    role: MessageRole
    user_id: str = ""
    images: List[str] = field(default_factory=list)
    audios: List[str] = field(default_factory=list)
    videos: List[str] = field(default_factory=list)
    files: List[str] = field(default_factory=list)
    created_at: Optional[str] = None
    updated_at: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Message":
        """Create a Message instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            chat_id=data.get("chatID", ""),
            text=data.get("text", ""),
            role=data.get("role", "user"),
            user_id=data.get("userID", ""),
            images=data.get("images", []),
            audios=data.get("audios", []),
            videos=data.get("videos", []),
            files=data.get("files", []),
            created_at=data.get("createdAt"),
            updated_at=data.get("updatedAt"),
        )


@dataclass
class MessageCreateData:
    """Data for creating a message."""

    chat_id: str
    text: str
    role: MessageRole = "user"
    user_id: str = ""
    images: List[str] = field(default_factory=list)
    audios: List[str] = field(default_factory=list)
    videos: List[str] = field(default_factory=list)
    files: List[str] = field(default_factory=list)


class Attachments(TypedDict, total=False):
    """Attachments for agent messages."""

    images: List[str]
    audios: List[str]
    videos: List[str]
    files: List[str]


@dataclass
class AgentMessage:
    """Message to send to an agent."""

    role: MessageRole
    content: str


@dataclass
class AgentRequest:
    """Request to send to an agent."""

    messages: List[AgentMessage]
    agentid: str
    chatid: Optional[str] = None
    raw_stream: bool = False
    phone_number: Optional[str] = None
    customer_id: Optional[str] = None
    provider_id: Optional[str] = None
    customer_info: Optional[Dict[str, Any]] = None


@dataclass
class AgentResponse:
    """Response from an agent."""

    chat_id: str
    message_id: str
    agent_response: Any

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "AgentResponse":
        """Create an AgentResponse instance from a dictionary."""
        return cls(
            chat_id=data.get("chatId", ""),
            message_id=data.get("messageId", ""),
            agent_response=data.get("agentResponse"),
        )


# Document types
@dataclass
class Document:
    """Represents a document."""

    id: str
    workspace_id: str = ""
    user_id: str = ""
    doc_type: DocumentType = "readComprehensionFile"
    title: str = ""
    topics: List[str] = field(default_factory=list)
    folder_id: str = ""
    external_path: str = ""
    source: str = ""
    status: str = ""
    scope: Optional[str] = None
    rawtext: Optional[str] = None
    created_at: Optional[str] = None
    updated_at: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Document":
        """Create a Document instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            workspace_id=_get_workspace_id(data),
            user_id=data.get("userid", ""),
            doc_type=data.get("type", "readComprehensionFile"),
            title=data.get("title", ""),
            topics=data.get("topics", []),
            folder_id=data.get("folderid", ""),
            external_path=data.get("external_path", ""),
            source=data.get("source", ""),
            status=data.get("status", ""),
            scope=data.get("scope"),
            rawtext=data.get("rawtext"),
            created_at=data.get("createdAt"),
            updated_at=data.get("updatedAt"),
        )


@dataclass
class DocumentCreateData:
    """Data for creating a document."""

    user_id: str
    title: str
    doc_type: DocumentType = "readComprehensionFile"
    topics: List[str] = field(default_factory=list)
    folder_id: str = "mrcRoot"
    external_path: str = ""
    source: str = ""
    status: str = "published"
    scope: Optional[str] = None


@dataclass
class DocumentUpdateData:
    """Data for updating a document."""

    title: Optional[str] = None
    topics: Optional[List[str]] = None
    folder_id: Optional[str] = None
    status: Optional[str] = None
    scope: Optional[str] = None


@dataclass
class FileUploadOptions:
    """Options for file upload."""

    folder_id: str = "mrcRoot"
    on_progress: Optional[Callable[[int, int, int], None]] = None


@dataclass
class Base64FileUploadOptions:
    """Options for base64 file upload."""

    filename: str
    content_type: str
    folder_id: str = "mrcRoot"
    on_progress: Optional[Callable[[int, int, int], None]] = None


@dataclass
class FileUploadResult:
    """Result of a file upload."""

    success: bool
    original_filename: str
    sanitized_filename: str
    filename: str
    import_type: str
    content_type: str
    size: int
    size_in_mb: float

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "FileUploadResult":
        """Create a FileUploadResult instance from a dictionary."""
        return cls(
            success=data.get("success", False),
            original_filename=data.get("originalFilename", ""),
            sanitized_filename=data.get("sanitizedFilename", ""),
            filename=data.get("filename", ""),
            import_type=data.get("importType", ""),
            content_type=data.get("contentType", ""),
            size=data.get("size", 0),
            size_in_mb=data.get("sizeInMB", 0.0),
        )


@dataclass
class FileDownloadOptions:
    """Options for file download."""

    context: str = "documents"
    on_progress: Optional[Callable[[int, int, int], None]] = None


@dataclass
class FileDownloadResult:
    """Result of a file download."""

    success: bool
    filename: str
    output_path: str
    size: int

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "FileDownloadResult":
        """Create a FileDownloadResult instance from a dictionary."""
        return cls(
            success=data.get("success", False),
            filename=data.get("filename", ""),
            output_path=data.get("outputPath", ""),
            size=data.get("size", 0),
        )


# Entity types
@dataclass
class Entity:
    """Represents an entity (topic, intent, or NER)."""

    id: str
    workspace_id: str = ""
    created_by: str = ""
    label: str = ""
    entity_type: EntityType = "topic"
    description: Optional[str] = None
    emoji: Optional[str] = None
    parent_entity: Optional[str] = None
    background_color: Optional[str] = None
    created_at: Optional[str] = None
    updated_at: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Entity":
        """Create an Entity instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            workspace_id=_get_workspace_id(data),
            created_by=data.get("createdBy", ""),
            label=data.get("label", ""),
            entity_type=data.get("type", "topic"),
            description=data.get("description"),
            emoji=data.get("emoji"),
            parent_entity=data.get("parentEntity"),
            background_color=data.get("backgroundColor"),
            created_at=data.get("createdAt"),
            updated_at=data.get("updatedAt"),
        )


@dataclass
class EntityCreateOptions:
    """Options for creating an entity."""

    description: Optional[str] = None
    emoji: Optional[str] = None
    parent_entity: Optional[str] = None
    background_color: Optional[str] = None


@dataclass
class EntityUpdateData:
    """Data for updating an entity."""

    label: Optional[str] = None
    description: Optional[str] = None
    emoji: Optional[str] = None
    parent_entity: Optional[str] = None
    background_color: Optional[str] = None


# Folder types
@dataclass
class Folder:
    """Represents a folder."""

    id: str
    workspace_id: str = ""
    created_by: str = ""
    name: str = ""
    description: Optional[str] = None
    emoji: Optional[str] = None
    status: Optional[str] = None
    parent: Optional[str] = None
    created_at: Optional[str] = None
    updated_at: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Folder":
        """Create a Folder instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            workspace_id=_get_workspace_id(data),
            created_by=data.get("createdBy", ""),
            name=data.get("name", ""),
            description=data.get("description"),
            emoji=data.get("emoji"),
            status=data.get("status"),
            parent=data.get("parent"),
            created_at=data.get("createdAt"),
            updated_at=data.get("updatedAt"),
        )


@dataclass
class FolderCreateOptions:
    """Options for creating a folder."""

    description: Optional[str] = None
    emoji: Optional[str] = None
    status: str = "active"
    parent: Optional[str] = None


@dataclass
class FolderUpdateData:
    """Data for updating a folder."""

    name: Optional[str] = None
    description: Optional[str] = None
    emoji: Optional[str] = None
    status: Optional[str] = None
    parent: Optional[str] = None


@dataclass
class FolderTreeNode:
    """A folder in a tree structure."""

    id: str
    workspace_id: str
    created_by: str
    name: str
    description: Optional[str]
    emoji: Optional[str]
    status: Optional[str]
    parent: Optional[str]
    created_at: Optional[str]
    updated_at: Optional[str]
    children: List["FolderTreeNode"] = field(default_factory=list)

    @classmethod
    def from_folder(
        cls, folder: Folder, children: Optional[List["FolderTreeNode"]] = None
    ) -> "FolderTreeNode":
        """Create a FolderTreeNode from a Folder."""
        return cls(
            id=folder.id,
            workspace_id=folder.workspace_id,
            created_by=folder.created_by,
            name=folder.name,
            description=folder.description,
            emoji=folder.emoji,
            status=folder.status,
            parent=folder.parent,
            created_at=folder.created_at,
            updated_at=folder.updated_at,
            children=children or [],
        )


# Prompt types (maps to CustomPrompt in GraphQL)
@dataclass
class Prompt:
    """Represents a custom prompt template."""

    id: str
    workspace_id: str = ""
    prompt_type: str = ""
    label: str = ""
    prompt_length: float = 0
    interpolation_string: str = ""
    scope: Optional[str] = None
    style: Optional[str] = None
    domain: Optional[str] = None
    prompt_placeholder: Optional[str] = None
    available_to_agents: List[str] = field(default_factory=list)
    created_by: Optional[str] = None
    updated_by: Optional[str] = None
    created_at: Optional[str] = None
    updated_at: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Prompt":
        """Create a Prompt instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            workspace_id=_get_workspace_id(data),
            prompt_type=data.get("type", ""),
            label=data.get("label", ""),
            prompt_length=data.get("promptLength", 0),
            interpolation_string=data.get("interpolationString", ""),
            scope=data.get("scope"),
            style=data.get("style"),
            domain=data.get("domain"),
            prompt_placeholder=data.get("promptPlaceholder"),
            available_to_agents=data.get("availableToAgents", []),
            created_by=data.get("createdBy"),
            updated_by=data.get("updatedBy"),
            created_at=data.get("createdAt"),
            updated_at=data.get("updatedAt"),
        )


# Alias for clarity
CustomPrompt = Prompt


@dataclass
class PromptCreateData:
    """Data for creating a prompt."""

    label: str
    prompt_type: str
    interpolation_string: str
    scope: Optional[str] = None
    style: Optional[str] = None
    domain: Optional[str] = None
    prompt_placeholder: Optional[str] = None
    available_to_agents: List[str] = field(default_factory=list)


@dataclass
class PromptUpdateData:
    """Data for updating a prompt."""

    label: Optional[str] = None
    prompt_type: Optional[str] = None
    interpolation_string: Optional[str] = None
    scope: Optional[str] = None
    style: Optional[str] = None
    domain: Optional[str] = None
    prompt_placeholder: Optional[str] = None
    available_to_agents: Optional[List[str]] = None


# Response types
@dataclass
class ListResponse:
    """Generic list response."""

    items: List[Any]
    total: Optional[int] = None
    limit: Optional[int] = None
    offset: Optional[int] = None


# Streaming types
class StreamingOptions(TypedDict, total=False):
    """Options for streaming requests."""

    chat_id: str
    phone_number: str
    customer_id: str
    provider_id: str
    customer_info: Dict[str, Any]
    attachments: Attachments
    show_progress: bool
    raw_stream: bool


@dataclass
class StreamEventData:
    """Data from a stream event."""

    text: str = ""
    message_type: str = ""
    message_id: str = ""
    chat_id: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "StreamEventData":
        """Create a StreamEventData instance from a dictionary."""
        return cls(
            text=data.get("text", ""),
            message_type=data.get("type", ""),
            message_id=data.get("message_id", ""),
            chat_id=data.get("chatId", ""),
        )


@dataclass
class StreamStatusEvent:
    """Status event from a stream."""

    status: str
    processing_status: Optional[str] = None
    metadata_parsed: Optional[Dict[str, Any]] = None


@dataclass
class StreamingResult:
    """Result of a streaming request."""

    chat_id: str
    message_id: str


# Content type mapping
CONTENT_TYPE_MAP: Dict[str, str] = {
    ".pdf": "application/pdf",
    ".doc": "application/msword",
    ".docx": "application/vnd.openxmlformats-officedocument.wordprocessingml.document",
    ".xls": "application/vnd.ms-excel",
    ".xlsx": "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
    ".ppt": "application/vnd.ms-powerpoint",
    ".pptx": "application/vnd.openxmlformats-officedocument.presentationml.presentation",
    ".txt": "text/plain",
    ".csv": "text/csv",
    ".json": "application/json",
    ".xml": "application/xml",
    ".html": "text/html",
    ".htm": "text/html",
    ".md": "text/markdown",
    ".rtf": "application/rtf",
    ".jpg": "image/jpeg",
    ".jpeg": "image/jpeg",
    ".png": "image/png",
    ".gif": "image/gif",
    ".bmp": "image/bmp",
    ".webp": "image/webp",
    ".svg": "image/svg+xml",
    ".mp3": "audio/mpeg",
    ".wav": "audio/wav",
    ".ogg": "audio/ogg",
    ".mp4": "video/mp4",
    ".webm": "video/webm",
    ".avi": "video/x-msvideo",
    ".mov": "video/quicktime",
    ".zip": "application/zip",
    ".rar": "application/vnd.rar",
    ".7z": "application/x-7z-compressed",
    ".tar": "application/x-tar",
    ".gz": "application/gzip",
    ".py": "text/x-python",
    ".js": "application/javascript",
    ".ts": "application/typescript",
    ".java": "text/x-java-source",
    ".c": "text/x-c",
    ".cpp": "text/x-c++",
    ".h": "text/x-c",
    ".hpp": "text/x-c++",
    ".cs": "text/x-csharp",
    ".go": "text/x-go",
    ".rs": "text/x-rust",
    ".rb": "text/x-ruby",
    ".php": "text/x-php",
    ".swift": "text/x-swift",
    ".kt": "text/x-kotlin",
    ".scala": "text/x-scala",
}


def get_content_type(filename: str) -> str:
    """Get content type from filename extension."""
    import os

    ext = os.path.splitext(filename)[1].lower()
    return CONTENT_TYPE_MAP.get(ext, "application/octet-stream")


# Max file size constant
MAX_FILE_SIZE_MB = 15
MAX_FILE_SIZE_BYTES = MAX_FILE_SIZE_MB * 1024 * 1024


@dataclass
class Agent:
    """Represents an AI agent."""

    id: str
    workspace_id: str = ""
    type: str = ""
    label: str = ""
    description: Optional[str] = None
    interpolation_string: Optional[str] = None
    emoji: Optional[str] = None
    color: Optional[str] = None
    dark_color: Optional[str] = None
    icon: Optional[str] = None
    large_logo: Optional[str] = None
    ico_url: Optional[str] = None
    logo_background: Optional[str] = None
    dark_logo_background: Optional[str] = None
    forced_theme: Optional[str] = None
    message_on_launch: Optional[str] = None
    messages_on_launch: List[str] = field(default_factory=list)
    rotate_messages_on_launch: bool = False
    rotate_messages_on_launch_interval: Optional[int] = None
    placeholder_message_on_loading: Optional[str] = None
    dynamic_placeholder_message_on_loading: bool = False
    custom_greeting_instructions: Optional[str] = None
    custom_conclusion_instructions: Optional[str] = None
    custom_charting_instructions: Optional[str] = None
    custom_summarisation_instructions: Optional[str] = None
    custom_images_instructions: Optional[str] = None
    custom_tooling_instructions: Optional[str] = None
    placeholder_input_message: Optional[str] = None
    open_by_default: bool = False
    agentic_coding_model: Optional[str] = None
    disclaimer: Optional[str] = None
    show_documents_references: bool = False
    is_default: bool = False
    has_memory: bool = True
    has_images: bool = False
    prompt_top_keywords: int = 5
    key_words_for_knowledge_base: bool = True
    has_topics_context: bool = True
    has_functions: bool = False
    advanced_language_detection: bool = False
    allowed_topics: Dict[str, Any] = field(default_factory=dict)
    static_docs: Dict[str, Any] = field(default_factory=dict)
    default_answer: Optional[str] = None
    no_knowledge_default_answer: Optional[str] = None
    quick_questions: Optional[str] = None
    recency_importance: Optional[float] = None
    max_recency_increase: Optional[float] = None
    recency_top_k_increase_factor: Optional[int] = None
    stop_sequence: Optional[str] = None
    examples: List[Any] = field(default_factory=list)
    enhancement_passage: Optional[str] = None
    inhibition_passage: Optional[str] = None
    pertinence_passage: Optional[str] = None
    goals: Optional[str] = None
    qa_url: Optional[str] = None
    top_k: int = 10
    mode: Optional[ChatMode] = None
    show_code_in_execution: bool = False
    hide_reasoning: bool = False
    charting: bool = False
    summarisation: bool = True
    compressor: bool = False
    doc_top_k: int = 5
    min_retrieval_score: Optional[float] = None
    time_significance: Optional[float] = None
    has_moderation: bool = False
    custom_moderations: Dict[str, Any] = field(default_factory=dict)
    moderation_message: Optional[str] = None
    language_detection_confidence: Optional[float] = None
    max_history: int = 10
    max_tokens: int = 1000
    topic_enhancer: bool = False
    max_topics: int = 3
    blender: bool = False
    temperature: float = 0.7
    version: int = 1
    subject: Optional[str] = None
    created_by: Optional[str] = None
    updated_by: Optional[str] = None
    agent_functions: List[str] = field(default_factory=list)
    is_global: bool = False
    prompt_buffering: bool = False
    function_context: Dict[str, Any] = field(default_factory=dict)
    plain_text_output: bool = False
    chain_of_thoughts: bool = False
    tree_of_thoughts: bool = False
    extended_output: bool = False
    agent_type: Optional[AgentType] = None
    agents_pool: List[str] = field(default_factory=list)
    mcps: List[str] = field(default_factory=list)
    max_planning_steps: Optional[int] = None
    max_browser_steps: Optional[int] = None
    virtual_desktop_instructions: Optional[str] = None
    planning_instructions: Optional[str] = None
    planner_execution_attempts: Optional[int] = None
    planner_requires_approval_for_plan_execution: bool = False
    planner_email_on_approval_request: bool = False
    planner_email_on_failed_plan: bool = False
    planner_email_on_completed_plan: bool = False
    planner_auto_agent_spawn: bool = False
    planner_retains_spawned_agents: bool = False
    planner_prompts_agents_programmatically: bool = False
    planner_deep_thinking: bool = False
    allow_replanning: bool = False
    planner_proceeds_when_improvements_required: bool = False
    planner_approver_emails: List[str] = field(default_factory=list)
    review_instructions: Optional[str] = None
    show_references_to_internal_docs: bool = False
    show_citations: bool = False
    force_json_output: bool = False
    agentic_rag: bool = False
    prevent_widget_usage: bool = False
    allow_feedback: bool = False
    generate_extended_report: bool = False
    show_time_for_response: bool = False
    show_detected_language: bool = False
    show_routed_model: bool = False
    is_multi_language_enabled: bool = False
    has_math: bool = False
    has_code: bool = False
    has_computer: bool = False
    has_ner: bool = False
    restricted_access: bool = False
    restricted_access_users: List[str] = field(default_factory=list)
    splash_title: Optional[str] = None
    show_splash_when_empty: bool = False
    llm_endpoint: Optional[str] = None
    llm_url: Optional[str] = None
    llm_url_status: Optional[QaUrlStatus] = None
    llm_url_type: Optional[QaUrlType] = None
    llm_base_model: Optional[str] = None
    llm_provider: Optional[str] = None
    llm_top_k: Optional[float] = None
    llm_seed: Optional[float] = None
    adverserial_provider: Optional[str] = None
    adverserial_model: Optional[str] = None
    function_calling_provider: Optional[str] = None
    function_calling_model: Optional[str] = None
    vision_provider: Optional[str] = None
    vision_model: Optional[str] = None
    video_generation_provider: Optional[str] = None
    video_generation_model: Optional[str] = None
    show_agent_name: bool = False
    allow_images_upload: bool = False
    allow_videos_upload: bool = False
    allow_audios_upload: bool = False
    allow_docs_upload: bool = False
    allow_images_generation: bool = False
    base_image_generation_model: Optional[str] = None
    max_images_generated: Optional[int] = None
    advanced_image_generation: bool = False
    allow_videos_generation: bool = False
    allow_audios_generation: bool = False
    allow_3d_model_generation: bool = False
    allow_introspection: bool = False
    voice_file_key: Optional[str] = None
    allow_internet_search: bool = False
    banned_domains: List[str] = field(default_factory=list)
    max_urls_for_internet_search: Optional[int] = None
    allow_deep_internet_search: bool = False
    internet_search_modes: List[InternetSearchMode] = field(default_factory=list)
    internet_search_location: Optional[str] = None
    auto_voice_mode: bool = False
    voice_sensitivity: Optional[str] = None
    keywords_weight_in_retrieval: Optional[float] = None
    custom_code_execution_environments: List[str] = field(default_factory=list)
    phone_number_as_sender: Optional[str] = None
    whatsapp_number_as_sender: Optional[str] = None
    email_as_sender: Optional[str] = None
    communication_services: List[str] = field(default_factory=list)
    channel_callback_delay: Optional[int] = None
    ephemeral_agent: bool = False
    parent_agent_id: Optional[str] = None
    parent_chat_id: Optional[str] = None
    minimum_subscription_type: Optional[PlanType] = None
    cloned_agent_id: Optional[str] = None
    cloned_agent_id_icon: Optional[str] = None
    has_content_creation: bool = False
    has_entity_extraction: bool = False
    department: Optional[Department] = None
    metadata: Dict[str, Any] = field(default_factory=dict)
    allow_email_to_agent: bool = False
    allowed_emails: Optional[str] = None
    agent_hiring_template: Dict[str, Any] = field(default_factory=dict)
    agent_desktop_vm: Optional[str] = None
    agent_desktop_port_range: Optional[str] = None
    agent_desktop_credentials: Dict[str, Any] = field(default_factory=dict)
    agent_desktop_browser: bool = False
    allow_3d_model_generation: bool = False
    agent_long_term_memory: Optional[str] = None
    is_long_term_memory_enabled: bool = False
    allow_external_api: bool = False
    force_new_thread_on_programmatic_request: bool = False
    dynamic_model_selection_instructions: Optional[str] = None
    context_relevancy_ratio: Optional[float] = None
    reasoning_budget: Optional[int] = None
    reasoning_mode: Optional[ReasoningMode] = None
    reasoning_effort: Optional[ReasoningEffort] = None
    fn_reasoning_effort: Optional[ReasoningEffort] = None
    agent_fn_params: Dict[str, Any] = field(default_factory=dict)
    canvas_enabled: bool = False
    advanced_compute: bool = False
    re_rank: bool = False
    min_deep_search_pages_percentage: Optional[float] = None
    voice_name: Optional[str] = None
    vad_thresholds: Optional[float] = None
    analytics_instructions: Optional[str] = None
    rag_instructions: Optional[str] = None
    allow_ask_question: bool = False
    rag_search_strategy: Optional[SearchStrategy] = None
    min_chunks_before_completeness: Optional[float] = None
    rag_completeness_threshold: Optional[float] = None
    voice_instructions: Optional[str] = None
    stt_model: Optional[str] = None
    llm_voice_model: Optional[str] = None
    tts_model: Optional[str] = None
    default_voice_language: Optional[str] = None
    voice_min_silence_duration: Optional[float] = None
    voice_min_endpointing_delay: Optional[float] = None
    voice_min_interruption_duration: Optional[float] = None
    enable_voice_intermissions: bool = False
    voice_background_noise: Optional[str] = None
    sip_phone_number: Optional[str] = None
    sip_dispatch_rule_id: Optional[str] = None
    enable_inbound_sip: bool = False
    enable_outbound_sip: bool = False
    enable_json_output: bool = False
    json_output_structure: Dict[str, Any] = field(default_factory=dict)
    predicted_plan_json: Dict[str, Any] = field(default_factory=dict)
    created_at: Optional[str] = None
    updated_at: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Agent":
        """Create an Agent instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            workspace_id=_get_workspace_id(data),
            type=data.get("type", ""),
            label=data.get("label", ""),
            description=data.get("description"),
            interpolation_string=data.get("interpolationString"),
            emoji=data.get("emoji"),
            color=data.get("color"),
            dark_color=data.get("darkColor"),
            icon=data.get("icon"),
            large_logo=data.get("largeLogo"),
            ico_url=data.get("icoUrl"),
            logo_background=data.get("logoBackground"),
            dark_logo_background=data.get("darkLogoBackground"),
            forced_theme=data.get("forcedTheme"),
            message_on_launch=data.get("messageOnLaunch"),
            messages_on_launch=data.get("messagesOnLaunch", []),
            rotate_messages_on_launch=data.get("rotateMessagesOnLaunch", False),
            rotate_messages_on_launch_interval=data.get("rotateMessagesOnLaunchInterval"),
            placeholder_message_on_loading=data.get("placeholderMessageOnLoading"),
            dynamic_placeholder_message_on_loading=data.get(
                "dynamicPlaceholderMessageOnLoading", False
            ),
            custom_greeting_instructions=data.get("customGreetingInstructions"),
            custom_conclusion_instructions=data.get("customConclusionInstructions"),
            custom_charting_instructions=data.get("customChartingInstructions"),
            custom_summarisation_instructions=data.get("customSummarisationInstructions"),
            custom_images_instructions=data.get("customImagesInstructions"),
            custom_tooling_instructions=data.get("customToolingInstructions"),
            placeholder_input_message=data.get("placeholderInputMessage"),
            open_by_default=data.get("openByDefault", False),
            agentic_coding_model=data.get("agenticCodingModel"),
            disclaimer=data.get("disclaimer"),
            show_documents_references=data.get("showDocumentsReferences", False),
            is_default=data.get("isDefault", False),
            has_memory=data.get("hasMemory", True),
            has_images=data.get("hasImages", False),
            prompt_top_keywords=data.get("promptTopKeywords", 5),
            key_words_for_knowledge_base=data.get("keyWordsForKnowledgeBase", True),
            has_topics_context=data.get("hasTopicsContext", True),
            has_functions=data.get("hasFunctions", False),
            advanced_language_detection=data.get("advancedLanguageDetection", False),
            allowed_topics=data.get("allowedTopics", {}),
            static_docs=data.get("staticDocs", {}),
            default_answer=data.get("defaultAnswer"),
            no_knowledge_default_answer=data.get("noKnowledgeDefaultAnswer"),
            quick_questions=data.get("quickQuestions"),
            recency_importance=data.get("recencyImportance"),
            max_recency_increase=data.get("maxRecencyIncrease"),
            recency_top_k_increase_factor=data.get("recencyTopKIncreaseFactor"),
            stop_sequence=data.get("stopSequence"),
            examples=data.get("examples", []),
            enhancement_passage=data.get("enhancementPassage"),
            inhibition_passage=data.get("inhibitionPassage"),
            pertinence_passage=data.get("pertinencePassage"),
            goals=data.get("goals"),
            qa_url=data.get("qaUrl"),
            top_k=data.get("topK", 10),
            mode=data.get("mode"),
            show_code_in_execution=data.get("showCodeInExecution", False),
            hide_reasoning=data.get("hideReasoning", False),
            charting=data.get("charting", False),
            summarisation=data.get("summarisation", True),
            compressor=data.get("compressor", False),
            doc_top_k=data.get("docTopK", 5),
            min_retrieval_score=data.get("minRetrievalScore"),
            time_significance=data.get("timeSignificance"),
            has_moderation=data.get("hasModeration", False),
            custom_moderations=data.get("customModerations", {}),
            moderation_message=data.get("moderationMessage"),
            language_detection_confidence=data.get("languageDetectionConfidence"),
            max_history=data.get("maxHistory", 10),
            max_tokens=data.get("maxTokens", 1000),
            topic_enhancer=data.get("topicEnhancer", False),
            max_topics=data.get("maxTopics", 3),
            blender=data.get("blender", False),
            temperature=data.get("temperature", 0.7),
            version=data.get("version", 1),
            subject=data.get("subject"),
            created_by=data.get("createdBy"),
            updated_by=data.get("updatedBy"),
            agent_functions=data.get("agentFunctions", []),
            is_global=data.get("isGlobal", False),
            prompt_buffering=data.get("promptBuffering", False),
            function_context=data.get("functionContext", {}),
            plain_text_output=data.get("plainTextOutput", False),
            chain_of_thoughts=data.get("chainOfThoughts", False),
            tree_of_thoughts=data.get("treeOfThoughts", False),
            extended_output=data.get("extendedOutput", False),
            agent_type=data.get("agentType"),
            agents_pool=data.get("agentsPool", []),
            mcps=data.get("mcps", []),
            max_planning_steps=data.get("maxPlanningSteps"),
            max_browser_steps=data.get("maxBrowserSteps"),
            virtual_desktop_instructions=data.get("virtualDesktopInstructions"),
            planning_instructions=data.get("planningInstructions"),
            planner_execution_attempts=data.get("plannerExecutionAttempts"),
            planner_requires_approval_for_plan_execution=data.get(
                "plannerRequiresApprovalForPlanExecution", False
            ),
            planner_email_on_approval_request=data.get("plannerEmailOnApprovalRequest", False),
            planner_email_on_failed_plan=data.get("plannerEmailOnFailedPlan", False),
            planner_email_on_completed_plan=data.get("plannerEmailOnCompletedPlan", False),
            planner_auto_agent_spawn=data.get("plannerAutoAgentSpawn", False),
            planner_retains_spawned_agents=data.get("plannerRetainsSpawnedAgents", False),
            planner_prompts_agents_programmatically=data.get(
                "plannerPromptsAgentsProgrammatically", False
            ),
            planner_deep_thinking=data.get("plannerDeepThinking", False),
            allow_replanning=data.get("allowReplanning", False),
            planner_proceeds_when_improvements_required=data.get(
                "plannerProceedsWhenImprovementsRequired", False
            ),
            planner_approver_emails=data.get("plannerApproverEmails", []),
            review_instructions=data.get("reviewInstructions"),
            show_references_to_internal_docs=data.get("showReferencesToInternalDocs", False),
            show_citations=data.get("showCitations", False),
            force_json_output=data.get("forceJsonOutput", False),
            agentic_rag=data.get("agenticRAG", False),
            prevent_widget_usage=data.get("preventWidgetUsage", False),
            allow_feedback=data.get("allowFeedback", False),
            generate_extended_report=data.get("generateExtendedReport", False),
            show_time_for_response=data.get("showTimeForResponse", False),
            show_detected_language=data.get("showDetectedLanguage", False),
            show_routed_model=data.get("showRoutedModel", False),
            is_multi_language_enabled=data.get("isMultiLanguageEnabled", False),
            has_math=data.get("hasMath", False),
            has_code=data.get("hasCode", False),
            has_computer=data.get("hasComputer", False),
            has_ner=data.get("hasNER", False),
            restricted_access=data.get("restrictedAccess", False),
            restricted_access_users=data.get("restrictedAccessUsers", []),
            splash_title=data.get("splashTitle"),
            show_splash_when_empty=data.get("showSplashWhenEmpty", False),
            llm_endpoint=data.get("llmEndpoint"),
            llm_url=data.get("llmUrl"),
            llm_url_status=data.get("llmUrlStatus"),
            llm_url_type=data.get("llmUrlType"),
            llm_base_model=data.get("llmBaseModel"),
            llm_provider=data.get("llmProvider"),
            llm_top_k=data.get("llmTopK"),
            llm_seed=data.get("llmSeed"),
            adverserial_provider=data.get("adverserialProvider"),
            adverserial_model=data.get("adverserialModel"),
            function_calling_provider=data.get("functionCallingProvider"),
            function_calling_model=data.get("functionCallingModel"),
            vision_provider=data.get("visionProvider"),
            vision_model=data.get("visionModel"),
            video_generation_provider=data.get("videoGenerationProvider"),
            video_generation_model=data.get("videoGenerationModel"),
            show_agent_name=data.get("showAgentName", False),
            allow_images_upload=data.get("allowImagesUpload", False),
            allow_videos_upload=data.get("allowVideosUpload", False),
            allow_audios_upload=data.get("allowAudiosUpload", False),
            allow_docs_upload=data.get("allowDocsUpload", False),
            allow_images_generation=data.get("allowImagesGeneration", False),
            base_image_generation_model=data.get("baseImageGenerationModel"),
            max_images_generated=data.get("maxImagesGenerated"),
            advanced_image_generation=data.get("advancedImageGeneration", False),
            allow_videos_generation=data.get("allowVideosGeneration", False),
            allow_audios_generation=data.get("allowAudiosGeneration", False),
            allow_3d_model_generation=data.get("allow3dModelGeneration", False),
            allow_introspection=data.get("allowIntrospection", False),
            voice_file_key=data.get("voiceFileKey"),
            allow_internet_search=data.get("allowInternetSearch", False),
            banned_domains=data.get("bannedDomains", []),
            max_urls_for_internet_search=data.get("maxUrlsForInternetSearch"),
            allow_deep_internet_search=data.get("allowDeepInternetSearch", False),
            internet_search_modes=data.get("internetSearchModes", []),
            internet_search_location=data.get("internetSearchLocation"),
            auto_voice_mode=data.get("autoVoiceMode", False),
            voice_sensitivity=data.get("voiceSensitivity"),
            keywords_weight_in_retrieval=data.get("keywordsWeightInRetrieval"),
            custom_code_execution_environments=data.get("customCodeExecutionEnvironments", []),
            phone_number_as_sender=data.get("phoneNumberAsSender"),
            whatsapp_number_as_sender=data.get("whatsappNumberAsSender"),
            email_as_sender=data.get("emailAsSender"),
            communication_services=data.get("communicationServices", []),
            channel_callback_delay=data.get("channelCallbackDelay"),
            ephemeral_agent=data.get("ephemeralAgent", False),
            parent_agent_id=data.get("parentAgentID"),
            parent_chat_id=data.get("parentChatID"),
            minimum_subscription_type=data.get("minimumSubscriptionType"),
            cloned_agent_id=data.get("clonedAgentID"),
            cloned_agent_id_icon=data.get("clonedAgentIDIcon"),
            has_content_creation=data.get("hasContentCreation", False),
            has_entity_extraction=data.get("hasEntityExtraction", False),
            department=data.get("department"),
            metadata=data.get("metadata", {}),
            allow_email_to_agent=data.get("allowEmailToAgent", False),
            allowed_emails=data.get("allowedEmails"),
            agent_hiring_template=data.get("agentHiringTemplate", {}),
            agent_desktop_vm=data.get("agentDesktopVM"),
            agent_desktop_port_range=data.get("agentDesktopPortRange"),
            agent_desktop_credentials=data.get("agentDesktopCredentials", {}),
            agent_desktop_browser=data.get("agentDesktopBrowser", False),
            agent_long_term_memory=data.get("agentLongTermMemory"),
            is_long_term_memory_enabled=data.get("isLongTermMemoryEnabled", False),
            allow_external_api=data.get("allowExternalAPI", False),
            force_new_thread_on_programmatic_request=data.get(
                "forceNewThreadOnProgrammaticRequest", False
            ),
            dynamic_model_selection_instructions=data.get("dynamicModelSelectionInstructions"),
            context_relevancy_ratio=data.get("contextRelevancyRatio"),
            reasoning_budget=data.get("reasoningBudget"),
            reasoning_mode=data.get("reasoningMode"),
            reasoning_effort=data.get("reasoningEffort"),
            fn_reasoning_effort=data.get("fnReasoningEffort"),
            agent_fn_params=data.get("agentFnParams", {}),
            canvas_enabled=data.get("canvasEnabled", False),
            advanced_compute=data.get("advancedCompute", False),
            re_rank=data.get("reRank", False),
            min_deep_search_pages_percentage=data.get("minDeepSearchPagesPercentage"),
            voice_name=data.get("voiceName"),
            vad_thresholds=data.get("vadThresholds"),
            analytics_instructions=data.get("analyticsInstructions"),
            rag_instructions=data.get("ragInstructions"),
            allow_ask_question=data.get("allowAskQuestion", False),
            rag_search_strategy=data.get("ragSearchStrategy"),
            min_chunks_before_completeness=data.get("minChunksBeforeCompleteness"),
            rag_completeness_threshold=data.get("ragCompletenessThreshold"),
            voice_instructions=data.get("voiceInstructions"),
            stt_model=data.get("sttModel"),
            llm_voice_model=data.get("llmVoiceModel"),
            tts_model=data.get("ttsModel"),
            default_voice_language=data.get("defaultVoiceLanguage"),
            voice_min_silence_duration=data.get("voiceMinSilenceDuration"),
            voice_min_endpointing_delay=data.get("voiceMinEndpointingDelay"),
            voice_min_interruption_duration=data.get("voiceMinInterruptionDuration"),
            enable_voice_intermissions=data.get("enableVoiceIntermissions", False),
            voice_background_noise=data.get("voiceBackgroundNoise"),
            sip_phone_number=data.get("sipPhoneNumber"),
            sip_dispatch_rule_id=data.get("sipDispatchRuleId"),
            enable_inbound_sip=data.get("enableInboundSip", False),
            enable_outbound_sip=data.get("enableOutboundSip", False),
            enable_json_output=data.get("enableJsonOutput", False),
            json_output_structure=data.get("jsonOutputStructure", {}),
            predicted_plan_json=data.get("predictedPlanJson", {}),
            created_at=data.get("createdAt"),
            updated_at=data.get("updatedAt"),
        )


# Agent Function types
@dataclass
class AgentFunction:
    """Represents an agent function."""

    id: str
    name: str
    description: str = ""
    url: Optional[str] = None
    workspace_id: str = ""
    parameters: Dict[str, Any] = field(default_factory=dict)
    model: Optional[str] = None
    request_type: Optional[FunctionRequestType] = None
    authorisation_type: Optional[AuthorisationType] = None
    depends_on_before_execution: List[str] = field(default_factory=list)
    depends_on_after_execution: List[str] = field(default_factory=list)
    authorisation_key: Optional[str] = None
    authorisation_id: Optional[str] = None
    params_as_path: bool = False
    scope: Optional[FunctionScope] = None
    json_path_extractor: Optional[str] = None
    headers: Dict[str, Any] = field(default_factory=dict)
    static_args: Dict[str, Any] = field(default_factory=dict)
    custom_execution_code: Optional[str] = None
    is_mcp_server: bool = False
    is_callback_fn: bool = False
    is_database_script: bool = False
    is_html_fn: bool = False
    is_chat_fn: bool = False
    is_hand_off_fn: bool = False
    is_code_template: bool = False
    is_agent_skill: bool = False
    code_execution_environment_id: Optional[str] = None
    on_fn_call: Dict[str, Any] = field(default_factory=dict)
    hand_off_agent: Optional[str] = None
    hand_off_to_original_agent_on_completion: bool = False
    hand_off_to_original_agent_on_failure: bool = False
    hand_off_agent_on_completion: Optional[str] = None
    hand_off_agent_on_failure: Optional[str] = None
    agent_skill_script: Optional[str] = None
    chat_script: Optional[str] = None
    chat_action: Optional[AgentFunctionChatAction] = None
    html_url: Optional[str] = None
    html_script: Optional[str] = None
    is_html_redirect_fn: bool = False
    html_form_on_successful_submit_message: Optional[str] = None
    html_form_on_failed_submit_message: Optional[str] = None
    db_schema_metadata: Dict[str, Any] = field(default_factory=dict)
    pre_execution_script: Optional[str] = None
    post_execution_script: Optional[str] = None
    db_script: Optional[str] = None
    db_connection_id: Optional[str] = None
    pre_execution_fn_id: Optional[str] = None
    pre_execution_mapping: Dict[str, Any] = field(default_factory=dict)
    oauth_connection_id: Optional[str] = None
    handed_off_to_human_on_call: bool = False
    graphql_query: Optional[str] = None
    graphql_mutation: Optional[str] = None
    graphql_operation_name: Optional[str] = None
    users_to_hand_off_to: List[str] = field(default_factory=list)
    created_by: Optional[str] = None
    updated_by: Optional[str] = None
    is_global: bool = False
    created_at: Optional[str] = None
    updated_at: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "AgentFunction":
        """Create an AgentFunction instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            name=data.get("name", ""),
            description=data.get("description", ""),
            url=data.get("url"),
            workspace_id=_get_workspace_id(data),
            parameters=data.get("parameters", {}),
            model=data.get("model"),
            request_type=data.get("requestType"),
            authorisation_type=data.get("authorisationType"),
            depends_on_before_execution=data.get("dependsOnBeforeExecution", []),
            depends_on_after_execution=data.get("dependsOnAfterExecution", []),
            authorisation_key=data.get("authorisationKey"),
            authorisation_id=data.get("authorisationID"),
            params_as_path=data.get("paramsAsPath", False),
            scope=data.get("scope"),
            json_path_extractor=data.get("jsonPathExtractor"),
            headers=data.get("headers", {}),
            static_args=data.get("staticArgs", {}),
            custom_execution_code=data.get("customExecutionCode"),
            is_mcp_server=data.get("isMCPServer"),
            is_callback_fn=data.get("isCallbackFn"),
            is_database_script=data.get("isDatabaseScript", False),
            is_html_fn=data.get("isHtmlFn"),
            is_chat_fn=data.get("isChatFn"),
            is_hand_off_fn=data.get("isHandOffFn"),
            is_code_template=data.get("isCodeTemplate"),
            is_agent_skill=data.get("isAgentSkill"),
            code_execution_environment_id=data.get("codeExecutionEnvironmentID"),
            on_fn_call=data.get("onFnCall", {}),
            hand_off_agent=data.get("handOffAgent"),
            hand_off_to_original_agent_on_completion=data.get("handOffToOriginalAgentOnCompletion"),
            hand_off_to_original_agent_on_failure=data.get("handOffToOriginalAgentOnFailure"),
            hand_off_agent_on_completion=data.get("handOffAgentOnCompletion"),
            hand_off_agent_on_failure=data.get("handOffAgentOnFailure"),
            agent_skill_script=data.get("agentSkillScript"),
            chat_script=data.get("chatScript"),
            chat_action=data.get("chatAction"),
            html_url=data.get("htmlUrl"),
            html_script=data.get("htmlScript"),
            is_html_redirect_fn=data.get("isHtmlRedirectFn"),
            html_form_on_successful_submit_message=data.get("htmlFormOnSuccessfulSubmitMessage"),
            html_form_on_failed_submit_message=data.get("htmlFormOnFailedSubmitMessage"),
            db_schema_metadata=data.get("dbSchemaMetadata", {}),
            pre_execution_script=data.get("preExecutionScript"),
            post_execution_script=data.get("postExecutionScript"),
            db_script=data.get("dbScript"),
            db_connection_id=data.get("dbConnectionID"),
            pre_execution_fn_id=data.get("preExecutionFnID"),
            pre_execution_mapping=data.get("preExecutionMapping", {}),
            oauth_connection_id=data.get("OAuthConnectionID"),
            handed_off_to_human_on_call=data.get("handedOffToHumanOnCall"),
            graphql_query=data.get("graphqlQuery"),
            graphql_mutation=data.get("graphqlMutation"),
            graphql_operation_name=data.get("graphqlOperationName"),
            users_to_hand_off_to=data.get("usersToHandOffTo", []),
            created_by=data.get("createdBy"),
            updated_by=data.get("updatedBy"),
            is_global=data.get("isGlobal", False),
            created_at=data.get("createdAt"),
            updated_at=data.get("updatedAt"),
        )


# Authorisation types
@dataclass
class Authorisation:
    """Represents an authorisation configuration."""

    id: str
    name: str
    type: str
    workspace_id: str = ""
    token_secret: Optional[str] = None
    description: Optional[str] = None
    scope: Optional[str] = None
    grant_type: Optional[str] = None
    client_id: Optional[str] = None
    client_secret: Optional[str] = None
    authorization_base_url: Optional[str] = None
    static_args: Optional[str] = None
    token_base_url: Optional[str] = None
    created_by: Optional[str] = None
    updated_by: Optional[str] = None
    created_at: Optional[str] = None
    updated_at: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Authorisation":
        """Create an Authorisation instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            name=data.get("name", ""),
            type=data.get("type", ""),
            workspace_id=_get_workspace_id(data),
            token_secret=data.get("tokenSecret"),
            description=data.get("description"),
            scope=data.get("scope"),
            grant_type=data.get("grantType"),
            client_id=data.get("clientId"),
            client_secret=data.get("clientSecret"),
            authorization_base_url=data.get("authorizationBaseUrl"),
            static_args=data.get("staticArgs"),
            token_base_url=data.get("tokenBaseUrl"),
            created_by=data.get("createdBy"),
            updated_by=data.get("updatedBy"),
            created_at=data.get("createdAt"),
            updated_at=data.get("updatedAt"),
        )


# Channel types
@dataclass
class Channel:
    """Represents a communication channel."""

    id: str
    name: str
    channel: str
    provider: str
    workspace_id: str = ""
    description: Optional[str] = None
    senderid: Optional[str] = None
    is_active: bool = True

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Channel":
        """Create a Channel instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            name=data.get("name", ""),
            channel=data.get("channel", ""),
            provider=data.get("provider", ""),
            workspace_id=_get_workspace_id(data),
            description=data.get("description"),
            senderid=data.get("senderid"),
            is_active=data.get("isActive", True),
        )


# Hosting types (maps to /connection/* API endpoints)
@dataclass
class Hosting:
    """Represents a hosting configuration for LLM/API endpoints."""

    id: str
    name: str
    type: HostingType
    workspace_id: str = ""
    description: Optional[str] = None
    token_secret: Optional[str] = None
    endpoint: Optional[str] = None
    model_name: Optional[str] = None
    base_model: Optional[str] = None
    custom_model: bool = False
    api_version: Optional[str] = None
    created_by: Optional[str] = None
    updated_by: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Hosting":
        """Create a Hosting instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            name=data.get("name", ""),
            type=data.get("type", "openai"),
            workspace_id=_get_workspace_id(data),
            description=data.get("description"),
            token_secret=data.get("tokenSecret"),
            endpoint=data.get("endpoint"),
            model_name=data.get("modelName"),
            base_model=data.get("baseModel"),
            custom_model=data.get("customModel", False),
            api_version=data.get("apiVersion"),
            created_by=data.get("createdBy"),
            updated_by=data.get("updatedBy"),
        )


# Alias for backward compatibility
Connection = Hosting


# Database Connection types
@dataclass
class DatabaseConnection:
    """Represents a database connection."""

    id: str
    name: str
    type: str
    host: str
    port: str
    username: str
    password_secret: str
    database: str
    workspace_id: str = ""
    description: Optional[str] = None
    schema: Optional[str] = None
    ssl: bool = False
    ssh: bool = False
    ssh_host: Optional[str] = None
    ssh_port: Optional[str] = None
    ssh_username: Optional[str] = None
    ssh_password_secret: Optional[str] = None
    created_by: Optional[str] = None
    updated_by: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "DatabaseConnection":
        """Create a DatabaseConnection instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            name=data.get("name", ""),
            type=data.get("type", ""),
            host=data.get("host", ""),
            port=data.get("port", ""),
            username=data.get("username", ""),
            password_secret=data.get("passwordSecret", ""),
            database=data.get("database", ""),
            workspace_id=_get_workspace_id(data),
            description=data.get("description"),
            schema=data.get("schema"),
            ssl=data.get("ssl", False),
            ssh=data.get("ssh", False),
            ssh_host=data.get("sshHost"),
            ssh_port=data.get("sshPort"),
            ssh_username=data.get("sshUsername"),
            ssh_password_secret=data.get("sshPasswordSecret"),
            created_by=data.get("createdBy"),
            updated_by=data.get("updatedBy"),
        )


# Member types
@dataclass
class Member:
    """Represents a workspace member."""

    id: str
    user_id: str
    workspace_id: str = ""
    role: str = "member"
    status: str = "active"
    created_by: Optional[str] = None
    updated_by: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Member":
        """Create a Member instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            user_id=data.get("userID", ""),
            workspace_id=_get_workspace_id(data),
            role=data.get("role", "member"),
            status=data.get("status", "active"),
            created_by=data.get("createdBy"),
            updated_by=data.get("updatedBy"),
        )


# Site types
@dataclass
class Site:
    """Represents a website for crawling."""

    id: str
    name: str
    url: str
    workspace_id: str = ""
    description: Optional[str] = None
    topics: Dict[str, Any] = field(default_factory=dict)
    status: str = "active"
    last_validation_date: Optional[str] = None
    validation_token: Optional[str] = None
    allowed_paths: List[str] = field(default_factory=list)
    completion_percentage: float = 0.0

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Site":
        """Create a Site instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            name=data.get("name", ""),
            url=data.get("url", ""),
            workspace_id=_get_workspace_id(data),
            description=data.get("description"),
            topics=data.get("topics", {}),
            status=data.get("status", "active"),
            last_validation_date=data.get("lastValidationDate"),
            validation_token=data.get("validationToken"),
            allowed_paths=data.get("allowedPaths", []),
            completion_percentage=data.get("completion_percentage", 0.0),
        )


# Benchmark types
@dataclass
class Benchmark:
    """Represents a benchmark for testing agent performance."""

    id: str
    name: str
    workspace_id: str = ""
    description: Optional[str] = None
    questions: List[Dict[str, str]] = field(default_factory=list)
    files: List[str] = field(default_factory=list)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Benchmark":
        """Create a Benchmark instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            name=data.get("name", ""),
            workspace_id=_get_workspace_id(data),
            description=data.get("description"),
            questions=data.get("questions", []),
            files=data.get("files", []),
        )


# CustomCodeExecutionEnvironment types (Hook)
@dataclass
class CustomCodeExecutionEnvironment:
    """Represents a custom code execution environment (formerly Hook)."""

    id: str
    name: str
    description: Optional[str] = None
    function_name: Optional[str] = None
    available_libraries: Optional[str] = None
    custom_execution_instructions: Optional[str] = None
    custom_execution_code: Optional[str] = None
    custom_execution_secrets: Dict[str, Any] = field(default_factory=dict)
    static_docs: Dict[str, Any] = field(default_factory=dict)
    allow_external_api: bool = False
    hardcoded_script: bool = False
    is_template: bool = False
    workspace_id: str = ""
    created_by: Optional[str] = None
    updated_by: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "CustomCodeExecutionEnvironment":
        """Create a CustomCodeExecutionEnvironment instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            name=data.get("name", ""),
            description=data.get("description"),
            function_name=data.get("functionName"),
            available_libraries=data.get("availableLibraries"),
            custom_execution_instructions=data.get("customExecutionInstructions"),
            custom_execution_code=data.get("customExecutionCode"),
            custom_execution_secrets=data.get("customExecutionSecrets", {}),
            static_docs=data.get("staticDocs", {}),
            allow_external_api=data.get("allowExternalAPI", False),
            hardcoded_script=data.get("hardcodedScript", False),
            is_template=data.get("isTemplate", False),
            workspace_id=_get_workspace_id(data),
            created_by=data.get("createdBy"),
            updated_by=data.get("updatedBy"),
        )


# Alias for backward compatibility
Hook = CustomCodeExecutionEnvironment


# CustomExecutionEnvironment types (remote execution environment)
@dataclass
class CustomExecutionEnvironment:
    """Represents a custom remote execution environment."""

    id: str
    name: str
    workspace_id: str = ""
    description: Optional[str] = None
    function_name: Optional[str] = None
    available_libraries: Optional[str] = None
    ip: Optional[str] = None
    port: Optional[str] = None
    metadata: Dict[str, Any] = field(default_factory=dict)
    is_global: bool = False
    created_by: Optional[str] = None
    updated_by: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "CustomExecutionEnvironment":
        """Create a CustomExecutionEnvironment instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            name=data.get("name", ""),
            workspace_id=_get_workspace_id(data),
            description=data.get("description"),
            function_name=data.get("functionName"),
            available_libraries=data.get("availableLibraries"),
            ip=data.get("ip"),
            port=data.get("port"),
            metadata=data.get("metadata", {}),
            is_global=data.get("isGlobal", False),
            created_by=data.get("createdBy"),
            updated_by=data.get("updatedBy"),
        )


# CronSchedule type
@dataclass
class CronSchedule:
    """Represents a cron schedule configuration."""

    frequency: CronJobFrequency = "DAILY"
    interval: Optional[int] = None
    cron_expression: Optional[str] = None
    day_of_week: Optional[int] = None
    day_of_month: Optional[int] = None
    hour: Optional[int] = None
    minute: Optional[int] = None
    start_date: Optional[str] = None
    end_date: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "CronSchedule":
        """Create a CronSchedule instance from a dictionary."""
        return cls(
            frequency=data.get("frequency", "DAILY"),
            interval=data.get("interval"),
            cron_expression=data.get("cronExpression"),
            day_of_week=data.get("dayOfWeek"),
            day_of_month=data.get("dayOfMonth"),
            hour=data.get("hour"),
            minute=data.get("minute"),
            start_date=data.get("startDate"),
            end_date=data.get("endDate"),
        )


# CronExecutionConfig type
@dataclass
class CronExecutionConfig:
    """Represents cron job execution configuration."""

    timeout: Optional[int] = None
    max_duration: Optional[int] = None
    priority: Optional[Priority] = None
    input_parameters: Dict[str, Any] = field(default_factory=dict)
    output_format: Optional[str] = None
    webhook_url: Optional[str] = None
    webhook_headers: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "CronExecutionConfig":
        """Create a CronExecutionConfig instance from a dictionary."""
        return cls(
            timeout=data.get("timeout"),
            max_duration=data.get("maxDuration"),
            priority=data.get("priority"),
            input_parameters=data.get("inputParameters", {}),
            output_format=data.get("outputFormat"),
            webhook_url=data.get("webhookUrl"),
            webhook_headers=data.get("webhookHeaders", {}),
        )


# CronNotificationConfig type
@dataclass
class CronNotificationConfig:
    """Represents cron job notification configuration."""

    on_success: bool = False
    on_failure: bool = False
    on_start: bool = False
    email_recipients: List[str] = field(default_factory=list)
    communication_service_id: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "CronNotificationConfig":
        """Create a CronNotificationConfig instance from a dictionary."""
        return cls(
            on_success=data.get("onSuccess", False),
            on_failure=data.get("onFailure", False),
            on_start=data.get("onStart", False),
            email_recipients=data.get("emailRecipients", []),
            communication_service_id=data.get("communicationServiceID"),
        )


# CronRetryConfig type
@dataclass
class CronRetryConfig:
    """Represents cron job retry configuration."""

    max_retries: int = 3
    retry_interval: int = 60
    backoff_multiplier: float = 1.5
    max_retry_interval: int = 3600

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "CronRetryConfig":
        """Create a CronRetryConfig instance from a dictionary."""
        return cls(
            max_retries=data.get("maxRetries", 3),
            retry_interval=data.get("retryInterval", 60),
            backoff_multiplier=data.get("backoffMultiplier", 1.5),
            max_retry_interval=data.get("maxRetryInterval", 3600),
        )


# Scheduled Job types (CronJob)
@dataclass
class ScheduledJob:
    """Represents a scheduled job (cron job)."""

    id: str
    name: str
    workspace_id: str = ""
    description: Optional[str] = None
    agent_id: Optional[str] = None
    custom_prompt_id: Optional[str] = None
    forced_prompt: Optional[str] = None
    schedule: Optional[CronSchedule] = None
    timezone: str = "UTC"
    is_active: bool = True
    last_run_at: Optional[str] = None
    next_run_at: Optional[str] = None
    status: CronJobStatus = "PENDING"
    execution_config: Optional[CronExecutionConfig] = None
    notification_config: Optional[CronNotificationConfig] = None
    retry_config: Optional[CronRetryConfig] = None
    metadata: Dict[str, Any] = field(default_factory=dict)
    start_date: Optional[str] = None
    end_date: Optional[str] = None
    created_by: Optional[str] = None
    updated_by: Optional[str] = None
    created_at: Optional[str] = None
    updated_at: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ScheduledJob":
        """Create a ScheduledJob instance from a dictionary."""
        schedule = None
        if data.get("schedule"):
            schedule = CronSchedule.from_dict(data["schedule"])

        execution_config = None
        if data.get("executionConfig"):
            execution_config = CronExecutionConfig.from_dict(data["executionConfig"])

        notification_config = None
        if data.get("notificationConfig"):
            notification_config = CronNotificationConfig.from_dict(data["notificationConfig"])

        retry_config = None
        if data.get("retryConfig"):
            retry_config = CronRetryConfig.from_dict(data["retryConfig"])

        return cls(
            id=data.get("id", ""),
            name=data.get("name", ""),
            workspace_id=_get_workspace_id(data),
            description=data.get("description"),
            agent_id=data.get("agentID"),
            custom_prompt_id=data.get("customPromptID"),
            forced_prompt=data.get("forcedPrompt"),
            schedule=schedule,
            timezone=data.get("timezone", "UTC"),
            is_active=data.get("isActive", True),
            last_run_at=data.get("lastRunAt"),
            next_run_at=data.get("nextRunAt"),
            status=data.get("status", "PENDING"),
            execution_config=execution_config,
            notification_config=notification_config,
            retry_config=retry_config,
            metadata=data.get("metadata", {}),
            start_date=data.get("startDate"),
            end_date=data.get("endDate"),
            created_by=data.get("createdBy"),
            updated_by=data.get("updatedBy"),
            created_at=data.get("createdAt"),
            updated_at=data.get("updatedAt"),
        )


# Alias for clarity
CronJob = ScheduledJob


# Secret types
@dataclass
class Secret:
    """Represents a secret."""

    id: str
    name: str
    workspace_id: str = ""
    description: Optional[str] = None
    created_at: Optional[str] = None
    updated_at: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Secret":
        """Create a Secret instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            name=data.get("name", ""),
            workspace_id=_get_workspace_id(data),
            description=data.get("description"),
            created_at=data.get("createdAt"),
            updated_at=data.get("updatedAt"),
        )


# Dictionary types
@dataclass
class DictionaryEntry:
    """Represents a dictionary entry for translations."""

    id: str
    owner: str
    source_language: str
    target_language: str
    source_text: str
    target_text: str
    workspace_id: str = ""
    pos: Optional[str] = None
    description: Optional[str] = None
    created_by: Optional[str] = None
    updated_by: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "DictionaryEntry":
        """Create a DictionaryEntry instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            owner=data.get("owner", ""),
            source_language=data.get("sourceLanguage", ""),
            target_language=data.get("targetLanguage", ""),
            source_text=data.get("sourceText", ""),
            target_text=data.get("targetText", ""),
            workspace_id=_get_workspace_id(data),
            pos=data.get("pos"),
            description=data.get("description"),
            created_by=data.get("createdBy"),
            updated_by=data.get("updatedBy"),
        )


# Embeddings types
@dataclass
class Embedding:
    """Represents a document embedding."""

    id: str
    chunk_id: str
    title: Optional[str] = None
    external_path: Optional[str] = None
    type: Optional[str] = None
    raw_text: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Embedding":
        """Create an Embedding instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            chunk_id=data.get("chunk_id", ""),
            title=data.get("title"),
            external_path=data.get("external_path"),
            type=data.get("type"),
            raw_text=data.get("raw_text"),
        )


# Charting Settings types
@dataclass
class ChartingSettings:
    """Represents workspace charting settings."""

    id: str
    workspace_id: str = ""
    primary_color: Optional[str] = None
    secondary_color: Optional[str] = None
    tertiary_color: Optional[str] = None
    primary_text_color: Optional[str] = None
    primary_border_color: Optional[str] = None
    line_color: Optional[str] = None
    plot_primary_color: Optional[str] = None
    plot_secondary_color: Optional[str] = None
    plot_tertiary_color: Optional[str] = None
    xy_background_color: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ChartingSettings":
        """Create a ChartingSettings instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            workspace_id=_get_workspace_id(data),
            primary_color=data.get("primaryColor"),
            secondary_color=data.get("secondaryColor"),
            tertiary_color=data.get("tertiaryColor"),
            primary_text_color=data.get("primaryTextColor"),
            primary_border_color=data.get("primaryBorderColor"),
            line_color=data.get("lineColor"),
            plot_primary_color=data.get("plotPrimaryColor"),
            plot_secondary_color=data.get("plotSecondaryColor"),
            plot_tertiary_color=data.get("plotTertiaryColor"),
            xy_background_color=data.get("xyBackgroundColor"),
        )


# Embeddings Settings types
@dataclass
class EmbeddingsSettings:
    """Represents workspace embeddings settings."""

    id: str
    workspace_id: str = ""
    max_chunk_words: int = 500
    max_overlap_sentences: int = 2
    chunking_strategy: str = "keywords"
    image_extraction_strategy: str = "safe"
    min_image_width: int = 100
    min_image_height: int = 100
    aspect_ratio_min: float = 0.1
    aspect_ratio_max: float = 1.0

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "EmbeddingsSettings":
        """Create an EmbeddingsSettings instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            workspace_id=_get_workspace_id(data),
            max_chunk_words=data.get("maxChunkWords", 500),
            max_overlap_sentences=data.get("maxOverlapSentences", 2),
            chunking_strategy=data.get("chunkingStrategy", "keywords"),
            image_extraction_strategy=data.get("imageExtractionStrategy", "safe"),
            min_image_width=data.get("minImageWidth", 100),
            min_image_height=data.get("minImageHeight", 100),
            aspect_ratio_min=data.get("aspectRatioMin", 0.1),
            aspect_ratio_max=data.get("aspectRatioMax", 1.0),
        )


# Billing types
@dataclass
class MonthCostsResponse:
    """Represents monthly usage and cost breakdown."""

    api_usage: Dict[str, Any] = field(default_factory=dict)
    training_usage: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "MonthCostsResponse":
        """Create a MonthCostsResponse instance from a dictionary."""
        return cls(
            api_usage=data.get("apiUsage", {}),
            training_usage=data.get("trainingUsage", {}),
        )


# Stream types
@dataclass
class OutputStream:
    """Represents an output stream."""

    id: str
    workspace_id: str = ""
    content: Optional[str] = None
    type: Optional[str] = None
    status: Optional[str] = None
    scoring: Optional[float] = None
    metadata: Optional[Dict[str, Any]] = None
    created_by: Optional[str] = None
    updated_by: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "OutputStream":
        """Create an OutputStream instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            workspace_id=_get_workspace_id(data),
            content=data.get("content"),
            type=data.get("type"),
            status=data.get("status"),
            scoring=data.get("scoring"),
            metadata=data.get("metadata"),
            created_by=data.get("createdBy"),
            updated_by=data.get("updatedBy"),
        )


# Request Log types
@dataclass
class RequestLog:
    """Represents a request log."""

    id: str
    workspace_id: str = ""
    srt_key: Optional[str] = None
    type: Optional[str] = None
    status: Optional[str] = None
    words: Optional[int] = None
    tokens: Optional[int] = None
    sentences: Optional[int] = None
    created_by: Optional[str] = None
    updated_by: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "RequestLog":
        """Create a RequestLog instance from a dictionary."""
        return cls(
            id=data.get("id", ""),
            workspace_id=_get_workspace_id(data),
            srt_key=data.get("srtKey"),
            type=data.get("type"),
            status=data.get("status"),
            words=data.get("words"),
            tokens=data.get("tokens"),
            sentences=data.get("sentences"),
            created_by=data.get("createdBy"),
            updated_by=data.get("updatedBy"),
        )



"""Field mappings for Agent type.

Maps SDK snake_case → GraphQL/API camelCase with exceptions.
Only exception cases are listed here - standard conversions use to_camel_case().

Derived from: /app/app/amplify/backend/api/toothfairyai/schema.graphql
"""

AGENT_FIELD_MAPPINGS = {
    "workspace_id": "workspaceid",
    
    "parent_agent_id": "parentAgentID",
    "parent_chat_id": "parentChatID",
    "cloned_agent_id": "clonedAgentID",
    
    "sip_dispatch_rule_id": "sipDispatchRuleId",
    
    "has_ner": "hasNER",
    "agentic_rag": "agenticRAG",
    "allow_external_api": "allowExternalAPI",
    "is_mfa_enabled": "isMFAEnabled",
}

AGENT_EXPLICIT_PARAMS = {
    "label",
    "description",
    "temperature",
    "max_tokens",
    "max_history",
    "top_k",
    "doc_top_k",
    "charting",
}